/* 1967 Steam Properties (Industrial Formulation)
   developed by Meyer, McClintock, Silvestri, and Spencer
   implemented by Dudley J. Benton */

#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <float.h>
#define _USE_MATH_DEFINES
#include <math.h>

#define Tc   647.3
#define Pc   22.12
#define rhoc 0.315457

static double BetaK(double theta)
  {
  static double k1=-7.691234564E+0; /* coefficients from Section 7.1.5, pg. 22 */
  static double k2=-2.608023696E+1; /* accompanying formula from Section 5., pg. 17 */
  static double k3=-1.681706546E+2;
  static double k4= 6.423285504E+1;
  static double k5=-1.189646225E+2;
  static double k6= 4.167117320E+0;
  static double k7= 2.097506760E+1;
  static double k8=1E9;
  static double k9=6;
  return(exp(
     ( k1*    (1.-theta)
      +k2*pow(1.-theta,2)
      +k3*pow(1.-theta,3)
      +k4*pow(1.-theta,4)
      +k5*pow(1.-theta,5))/theta/(1.+k6*(1.-theta)+k7*pow(1.-theta,2))
    -(1.-theta)/(k8*pow(1.-theta,2)+k9)
    ));
  }

static double BetaL(double theta)/* Region 2/3 boundary equation (dimensionless)*/
  {
  static double L     =7.160997524E+0; /* defined in Section 7.1.6, pg. 22 */
  static double theta1=9.626911787E-1; /* constants in Section 8.1, pg. 23 */
  static double theta2=1.333462073E+0;
  static double theta3=1.657886606E+0;
  static double beta1 =7.475191707E-1;
  static double beta2 =4.520795660E+0;
  return(((theta2-theta)*beta1+(theta-theta1)*beta2-L*(theta2-theta)*(theta-theta1))/(theta2-theta1));
  }

static double BetaLp(double theta) /* d(BetaL)/d(theta) */
  {
  static double L     =7.160997524E+0; /* defined in Section 7.1.6, pg. 22 */
  static double theta1=9.626911787E-1; /* constants in Section 8.1, pg. 23 */
  static double theta2=1.333462073E+0;
  static double theta3=1.657886606E+0;
  static double beta1 =7.475191707E-1;
  static double beta2 =4.520795660E+0;
  return((beta2-beta1-L*(theta2-2*theta+theta1))/(theta2-theta1));
  }

static double Chi1(double theta,double beta)
  {
  static double A0   =  6.824687741E+3;
  static double A11  =  7.982692717E+0; /* coefficients from Section 7.1; pg. 20 */
  static double A12  = -2.616571843E-2; /* accompanying formula from Section 9.1; pg. 24 */
  static double A13  =  1.522411790E-3;
  static double A14  =  2.284279054E-2;
  static double A15  =  2.421647003E+2;
  static double A16  =  1.269716088E-10;
  static double A17  =  2.074838328E-7;
  static double A18  =  2.174020350E-8;
  static double A19  =  1.105710498E-9;
  static double A20  =  1.293441934E+1;
  static double A21  =  1.308119072E-5;
  static double A22  =  6.047626338E-14;
  static double A[10]={-5.422063673E+2,
                       -2.096666205E+4,
                        3.941286787E+4,
                       -6.733277739E+4,
                        9.902381028E+4,
                       -1.093911774E+5,
                        8.590841667E+4,
                       -4.511168742E+4,
                        1.418138926E+4,
                       -2.017271113E+3};
  static double a1 = 8.438375405E-1;
  static double a2 = 5.362162162E-4;
  static double a3 = 1.720000000E+0;
  static double a4 = 7.342278489E-2;
  static double a5 = 4.975858870E-2;
  static double a6 = 6.537154300E-1;
  static double a7 = 1.150000000E-6;
  static double a8 = 1.510800000E-5;
  static double a9 = 1.418800000E-1;
  static double a10= 7.002753165E+0;
  static double a11= 2.995284926E-4;
  static double a12= 2.040000000E-1;
  double Y,Z;
  Y=1.-a1*pow(theta,2)-a2*pow(theta,-6);
  Z=Y+sqrt(a3*pow(Y,2)-2.*a4*theta+2.*a5*beta);
  return(
    A11*a5*pow(Z,-5./17.)+(A12+A13*theta+A14*pow(theta,2)+A15*pow(a6-theta,10)+A16*pow(a7+pow(theta,19),-1))
    -pow(a8+pow(theta,11),-1)*(A17+2.*A18*beta+3.*A19*pow(beta,2))
    -A20*pow(theta,18)*(a9+pow(theta,2))*(-3.*pow(a10+beta,-4)+a11)+3.*A21*(a12-theta)*pow(beta,2)
    +4.*A22*pow(theta,-20)*pow(beta,3)
    );
  }

static double Epsilon1(double theta,double beta)
  {
  static double A0   =  6.824687741E+3;
  static double A11  =  7.982692717E+0; /* coefficients from Section 7.1; pg. 20 */
  static double A12  = -2.616571843E-2; /* accompanying formula from Section 9.1; pg. 24 */
  static double A13  =  1.522411790E-3;
  static double A14  =  2.284279054E-2;
  static double A15  =  2.421647003E+2;
  static double A16  =  1.269716088E-10;
  static double A17  =  2.074838328E-7;
  static double A18  =  2.174020350E-8;
  static double A19  =  1.105710498E-9;
  static double A20  =  1.293441934E+1;
  static double A21  =  1.308119072E-5;
  static double A22  =  6.047626338E-14;
  static double A[10]={-5.422063673E+2,
                     -2.096666205E+4,
                      3.941286787E+4,
                     -6.733277739E+4,
                      9.902381028E+4,
                     -1.093911774E+5,
                      8.590841667E+4,
                     -4.511168742E+4,
                      1.418138926E+4,
                     -2.017271113E+3};
  static double a1 = 8.438375405E-1;
  static double a2 = 5.362162162E-4;
  static double a3 = 1.720000000E+0;
  static double a4 = 7.342278489E-2;
  static double a5 = 4.975858870E-2;
  static double a6 = 6.537154300E-1;
  static double a7 = 1.150000000E-6;
  static double a8 = 1.510800000E-5;
  static double a9 = 1.418800000E-1;
  static double a10= 7.002753165E+0;
  static double a11= 2.995284926E-4;
  static double a12= 2.040000000E-1;
  static double alpha0=0; /* constants defined in Section 8.1, pg. 23 */
  int nu;
  double sum,Y,Yp,Z;
  Y=1.-a1*pow(theta,2)-a2*pow(theta,-6);
  Yp=-2.*a1*theta+6.*a2*pow(theta,-7);
  Z=Y+sqrt(a3*pow(Y,2)-2.*a4*theta+2.*a5*beta);
  sum=0.;
  for(nu=1;nu<=10;nu++)
    sum+=(nu-2)*A[nu-1]*pow(theta,nu-1);
  return(
    alpha0+A0*theta-sum+A11*(Z*(17.*(Z/29.-Y/12.)+5.*theta*Yp/12.)+a4*theta-(a3-1.)*theta*Y*Yp)*pow(Z,-5./17.)
    +(A12-A14*pow(theta,2)+A15*(9.*theta+a6)*pow(a6-theta,9)+A16*(20.*pow(theta,19)+a7)*pow(a7+pow(theta,19),-2))*beta
    -(12.*pow(theta,11)+a8)*pow(a8+pow(theta,11),-2)*(A17*beta+A18*pow(beta,2)+A19*pow(beta,3))+A20*pow(theta,18)*(17.*a9+19.*pow(theta,2))*(pow(a10+beta,-3)+a11*beta)
    +A21*a12*pow(beta,3)+21.*A22*pow(theta,-20)*pow(beta,4)
    );
  }

static double Sigma1(double theta,double beta)
  {
  static double A0   =  6.824687741E+3;
  static double A11  =  7.982692717E+0; /* coefficients from Section 7.1; pg. 20 */
  static double A12  = -2.616571843E-2; /* accompanying formula from Section 9.1; pg. 24 */
  static double A13  =  1.522411790E-3;
  static double A14  =  2.284279054E-2;
  static double A15  =  2.421647003E+2;
  static double A16  =  1.269716088E-10;
  static double A17  =  2.074838328E-7;
  static double A18  =  2.174020350E-8;
  static double A19  =  1.105710498E-9;
  static double A20  =  1.293441934E+1;
  static double A21  =  1.308119072E-5;
  static double A22  =  6.047626338E-14;
  static double A[10]={-5.422063673E+2,
                     -2.096666205E+4,
                      3.941286787E+4,
                     -6.733277739E+4,
                      9.902381028E+4,
                     -1.093911774E+5,
                      8.590841667E+4,
                     -4.511168742E+4,
                      1.418138926E+4,
                     -2.017271113E+3};
  static double a1 = 8.438375405E-1;
  static double a2 = 5.362162162E-4;
  static double a3 = 1.720000000E+0;
  static double a4 = 7.342278489E-2;
  static double a5 = 4.975858870E-2;
  static double a6 = 6.537154300E-1;
  static double a7 = 1.150000000E-6;
  static double a8 = 1.510800000E-5;
  static double a9 = 1.418800000E-1;
  static double a10= 7.002753165E+0;
  static double a11= 2.995284926E-4;
  static double a12= 2.040000000E-1;
  static double alpha1=0;
  int nu;
  double sum,Y,Yp,Z;
  Y=1.-a1*pow(theta,2)-a2*pow(theta,-6);
  Yp=-2.*a1*theta+6.*a2*pow(theta,-7);
  Z=Y+sqrt(a3*pow(Y,2)-2.*a4*theta+2.*a5*beta);
  sum=0.;
  for(nu=2;nu<=10;nu++)
    sum+=(nu-1)*A[nu-1]*pow(theta,nu-2);
  return(
    -alpha1+A0*log(theta)-sum+A11*((5.*Z/12.-(a3-1.)*Y)*Yp+a4)*pow(Z,-5./17.)
    +(-A13-2*A14*theta+10.*A15*pow(a6-theta,9)+19.*A16*pow(a7+pow(theta,19),-2)*pow(theta,18))*beta
    -11.*pow(a8+pow(theta,11),-2)*pow(theta,10)*(A17*beta+A18*pow(beta,2)+A19*pow(beta,3))+A20*pow(theta,17)*(18.*a9+20*pow(theta,2))*(pow(a10+beta,-3)+a11*beta)
    +A21*pow(beta,3)+20.*A22*pow(theta,-21)*pow(beta,4)
    );
  }

static double Chi2(double theta,double beta)
  {
  static int n[8]={2,3,2,2,3,2,2,2}; /* These coefficients appear */
  static int z[8][3]={{13, 3, 0},    /* at the bottom of page 18. */
                      {18, 2, 1},
                      {18,10, 0},
                      {25,14, 0},
                      {32,28,24},
                      {12,11, 0},
                      {24,18, 0},
                      {24,14, 0}};
  static int l[3]={1,1,2};
  static int x[3][2]={{14, 0},
                      {19, 0},
                      {54,27}};
  static double B[10][7]={ /* coefficients defined in Section 7.1.2, pg. 20 */
    {1.683599274E+1, 2.856067796E+1,-5.438923329E+1, 4.330662834E-1,-6.547711697E-1, 8.565182058E-2,       0       },
    {      0       , 6.670375918E-2, 1.388983801E+0,       0       ,       0       ,       0       ,       0       },
    {      0       , 8.390104328E-2, 2.614670893E-2,-3.373439453E-2,       0       ,       0       ,       0       },
    {      0       , 4.520918904E-1, 1.069036614E-1,       0       ,       0       ,       0       ,       0       },
    {      0       ,-5.975336707E-1,-8.847535804E-2,       0       ,       0       ,       0       ,       0       },
    {      0       , 5.958051609E-1,-5.159303373E-1, 2.075021122E-1,       0       ,       0       ,       0       },
    {      0       , 1.190610271E-1,-9.867174132E-2,       0       ,       0       ,       0       ,       0       },
    {      0       , 1.683998803E-1,-5.809438001E-2,       0       ,       0       ,       0       ,       0       },
    {      0       , 6.552390126E-3, 5.710218649E-4,       0       ,       0       ,       0       ,       0       },
    {1.936587558E+2,-1.388522425E+3, 4.126607219E+3,-6.508211677E+3, 5.745984054E+3,-2.693088365E+3, 5.235718623E+2}};
  static double bb[3][2]={ /* coefficients defined in Section 7.1.2, pg. 20 */
    { 4.006073948E-1,      0       },
    { 8.636081627E-2,      0       },
    {-8.532322921E-1,3.460208861E-1}};
  static double b     =7.633333333E-1; /* defined in section 7.1.2, pg. 20 */
  static double l1    =4.260321148E+0;
  int lambda,mu,nu;
  double betaL,sum1,sum2,sum3,sum4,sum5,sum6,X;
  X=exp(b*(1.-theta));
  betaL=BetaL(theta);
  sum1=0.;
  for(mu=1;mu<=5;mu++)
    {
    sum2=0.;
    for(nu=1;nu<=n[mu-1];nu++)
      sum2+=B[mu][nu]*pow(X,z[mu-1][nu-1]);
    sum1+=mu*pow(beta,mu-1)*sum2;
    }
  sum3=0.;
  for(mu=6;mu<=8;mu++)
    {
    sum4=0.;
    for(nu=1;nu<=n[mu-1];nu++)
      sum4+=B[mu][nu]*pow(X,z[mu-1][nu-1]);
    sum5=0.;
    for(lambda=1;lambda<=l[mu-6];lambda++)
      sum5+=bb[mu-6][lambda-1]*pow(X,x[mu-6][lambda-1]);
    sum3+=(mu-2)*pow(beta,1-mu)*sum4/pow(pow(beta,2-mu)+sum5,2);
    }
  sum6=0.;
  for(nu=0;nu<=6;nu++)
    sum6+=B[9][nu]*pow(X,nu);
  return(
     l1*theta/beta-sum1-sum3
    +11.*pow(beta/betaL,10)*sum6
    );
  }

static double Epsilon2(double theta,double beta)
  {
  static int n[8]={2,3,2,2,3,2,2,2}; /* These coefficients appear */
  static int z[8][3]={{13, 3, 0},    /* at the bottom of page 18. */
                      {18, 2, 1},
                      {18,10, 0},
                      {25,14, 0},
                      {32,28,24},
                      {12,11, 0},
                      {24,18, 0},
                      {24,14, 0}};
  static int l[3]={1,1,2};
  static int x[3][2]={{14, 0},
                      {19, 0},
                      {54,27}};
  static double B[10][7]={ /* coefficients defined in Section 7.1.2, pg. 20 */
    {1.683599274E+1, 2.856067796E+1,-5.438923329E+1, 4.330662834E-1,-6.547711697E-1, 8.565182058E-2,       0       },
    {      0       , 6.670375918E-2, 1.388983801E+0,       0       ,       0       ,       0       ,       0       },
    {      0       , 8.390104328E-2, 2.614670893E-2,-3.373439453E-2,       0       ,       0       ,       0       },
    {      0       , 4.520918904E-1, 1.069036614E-1,       0       ,       0       ,       0       ,       0       },
    {      0       ,-5.975336707E-1,-8.847535804E-2,       0       ,       0       ,       0       ,       0       },
    {      0       , 5.958051609E-1,-5.159303373E-1, 2.075021122E-1,       0       ,       0       ,       0       },
    {      0       , 1.190610271E-1,-9.867174132E-2,       0       ,       0       ,       0       ,       0       },
    {      0       , 1.683998803E-1,-5.809438001E-2,       0       ,       0       ,       0       ,       0       },
    {      0       , 6.552390126E-3, 5.710218649E-4,       0       ,       0       ,       0       ,       0       },
    {1.936587558E+2,-1.388522425E+3, 4.126607219E+3,-6.508211677E+3, 5.745984054E+3,-2.693088365E+3, 5.235718623E+2}};
  static double bb[3][2]={ /* coefficients defined in Section 7.1.2, pg. 20 */
    { 4.006073948E-1,      0       },
    { 8.636081627E-2,      0       },
    {-8.532322921E-1,3.460208861E-1}};
  static double b     =7.633333333E-1; /* defined in section 7.1.2, pg. 20 */
  static double l1    =4.260321148E+0;
  static double alpha0=0; /* constants defined in Section 8.1, pg. 23 */
  int lambda,mu,nu;
  double betaL,betaLp,sum1,sum2,sum3,sum4,sum5,sum6,sum7,sum8,X;
  X=exp(b*(1.-theta));
  betaL=BetaL(theta);
  betaLp=BetaLp(theta);
  sum1=0.;
  for(nu=1;nu<=5;nu++)
    sum1+=(nu-2)*B[0][nu]*pow(theta,nu-1);
  sum2=0.;
  for(mu=1;mu<=5;mu++)
    {
    sum3=0.;
    for(nu=1;nu<=n[mu-1];nu++)
      sum3+=(1.+z[mu-1][nu-1]*b*theta)*B[mu][nu]*pow(X,z[mu-1][nu-1]);
    sum2+=pow(beta,mu)*sum3;
    }
  sum4=0.;
  for(mu=6;mu<=8;mu++)
    {
    sum6=0.;
    for(lambda=1;lambda<=l[mu-6];lambda++)
      sum6+=x[mu-6][lambda-1]*bb[mu-6][lambda-1]*pow(X,x[mu-6][lambda-1]);
    sum7=0.;
    for(lambda=1;lambda<=l[mu-6];lambda++)
      sum7+=bb[mu-6][lambda-1]*pow(X,x[mu-6][lambda-1]);
    sum5=0.;
    for(nu=1;nu<=n[mu-1];nu++)
      sum5+=B[mu][nu]*pow(X,z[mu-1][nu-1])*(1.+z[mu-1][nu-1]*b*theta-b*theta*sum6/(pow(beta,2-mu)+sum7));
    sum4+=sum5/(pow(beta,2-mu)+sum7);
    }
  sum8=0.;
  for(nu=0;nu<=6;nu++)
    sum8+=(1.+theta*(10*betaLp/betaL+nu*b))*B[9][nu]*pow(X,nu);
  return(
     alpha0+B[0][0]*theta-sum1-sum2
    -sum4
    +beta*pow(beta/betaL,10)*sum8
    );
  }

static double Sigma2(double theta,double beta)
  {
  static int n[8]={2,3,2,2,3,2,2,2}; /* These coefficients appear */
  static int z[8][3]={{13, 3, 0},    /* at the bottom of page 18. */
                      {18, 2, 1},
                      {18,10, 0},
                      {25,14, 0},
                      {32,28,24},
                      {12,11, 0},
                      {24,18, 0},
                      {24,14, 0}};
  static int l[3]={1,1,2};
  static int x[3][2]={{14, 0},
                      {19, 0},
                      {54,27}};
  static double B[10][7]={ /* coefficients defined in Section 7.1.2, pg. 20 */
    {1.683599274E+1, 2.856067796E+1,-5.438923329E+1, 4.330662834E-1,-6.547711697E-1, 8.565182058E-2,       0       },
    {      0       , 6.670375918E-2, 1.388983801E+0,       0       ,       0       ,       0       ,       0       },
    {      0       , 8.390104328E-2, 2.614670893E-2,-3.373439453E-2,       0       ,       0       ,       0       },
    {      0       , 4.520918904E-1, 1.069036614E-1,       0       ,       0       ,       0       ,       0       },
    {      0       ,-5.975336707E-1,-8.847535804E-2,       0       ,       0       ,       0       ,       0       },
    {      0       , 5.958051609E-1,-5.159303373E-1, 2.075021122E-1,       0       ,       0       ,       0       },
    {      0       , 1.190610271E-1,-9.867174132E-2,       0       ,       0       ,       0       ,       0       },
    {      0       , 1.683998803E-1,-5.809438001E-2,       0       ,       0       ,       0       ,       0       },
    {      0       , 6.552390126E-3, 5.710218649E-4,       0       ,       0       ,       0       ,       0       },
    {1.936587558E+2,-1.388522425E+3, 4.126607219E+3,-6.508211677E+3, 5.745984054E+3,-2.693088365E+3, 5.235718623E+2}};
  static double bb[3][2]={ /* coefficients defined in Section 7.1.2, pg. 20 */
    { 4.006073948E-1,      0       },
    { 8.636081627E-2,      0       },
    {-8.532322921E-1,3.460208861E-1}};
  static double b     =7.633333333E-1; /* defined in section 7.1.2, pg. 20 */
  static double l1    =4.260321148E+0;
  static double alpha1=0;
  int lambda,mu,nu;
  double betaL,betaLp,sum1,sum2,sum3,sum4,sum5,sum6,sum7,sum8,X;
  X=exp(b*(1.-theta));
  betaL=BetaL(theta);
  betaLp=BetaLp(theta);
  sum1=0.;
  for(nu=1;nu<=5;nu++)
    sum1+=(nu-1)*B[0][nu]*pow(theta,nu-2);
  sum2=0.;
  for(mu=1;mu<=5;mu++)
    {
    sum3=0.;
    for(nu=1;nu<=n[mu-1];nu++)
      sum3+=z[mu-1][nu-1]*B[mu][nu]*pow(X,z[mu-1][nu-1]);
    sum2+=pow(beta,mu)*sum3;
    }
  sum4=0.;
  for(mu=6;mu<=8;mu++)
    {
    sum6=0.;
    for(lambda=1;lambda<=l[mu-6];lambda++)
      sum6+=x[mu-6][lambda-1]*bb[mu-6][lambda-1]*pow(X,x[mu-6][lambda-1]);
    sum7=0.;
    for(lambda=1;lambda<=l[mu-6];lambda++)
      sum7+=bb[mu-6][lambda-1]*pow(X,x[mu-6][lambda-1]);
    sum5=0.;
    for(nu=1;nu<=n[mu-1];nu++)
      sum5+=B[mu][nu]*pow(X,z[mu-1][nu-1])*(z[mu-1][nu-1]-sum6/(pow(beta,2-mu)+sum7));
    sum4+=sum5/(pow(beta,2-mu)+sum7);
    }
  sum8=0.;
  for(nu=0;nu<=6;nu++)
    sum8+=(10*betaLp/betaL+nu*b)*B[9][nu]*pow(X,nu);
  return(
    -alpha1-l1*log(beta)+B[0][0]*log(theta)-sum1-b*sum2
    -b*sum4
    +beta*pow(beta/betaL,10)*sum8
    );
  }

static double Beta3(double theta,double chi)
  {
  static double C[8][13]={ /* coefficients from Section 7.1.3, pg. 21; equation from Section 9.3, pg. 26 */
    {-6.839900000E+0,-1.722604200E-2,-7.771750390E+0, 4.204607520E+0,-2.768070380E+0, 2.104197070E+0,-1.146495880E+0,
      2.231380850E-1, 1.162503630E-1,-8.209005440E-2, 1.941292390E-2,-1.694705760E-3,-4.311577033E+0},
    {       0       , 7.086360850E-1, 1.236794550E+1,-1.203890040E+1, 5.404374220E+0,-9.938650430E-1, 6.275231820E-2,
     -7.747430160E+0,       0       ,       0       ,       0       ,       0       ,       0       },
    {       0       ,-4.298850920E+0, 4.314305380E+1,-1.416193130E+1, 4.041724590E+0, 1.555463260E+0,-1.665689350E+0,
      3.248811580E-1, 2.936553250E+1,       0       ,       0       ,       0       ,       0       },
    {       0       , 7.948418420E-6, 8.088597470E+1,-8.361533800E+1, 3.586365170E+1, 7.518959540E+0,-1.261606400E+1,
      1.097174620E+0, 2.121454920E+0,-5.465295660E-1, 8.328754130E+0,       0       ,       0       },
    { 2.759717760E-6,-5.090739850E-4,       0       ,       0       ,       0       ,       0       ,       0       ,
            0       ,       0       ,       0       ,       0       ,       0       ,       0       },
    { 2.106363320E+2,       0       ,       0       ,       0       ,       0       ,       0       ,       0       ,
            0       ,       0       ,       0       ,       0       ,       0       ,       0       },
    { 5.528935335E-2,-2.336365955E-1, 3.697071420E-1,-2.596415470E-1, 6.828087013E-2,       0       ,       0       ,
            0       ,       0       ,       0       ,       0       ,       0       ,       0       },
    {-2.571600553E+2,-1.518783715E+2, 2.220723208E+1,-1.802039570E+2, 2.357096220E+3,-1.462335698E+4, 4.542916630E+4,
     -7.053556432E+4, 4.381571428E+4,       0       ,       0       ,       0       ,       0       }};
  int nu;
  double sum1,sum2,sum3,sum4,sum5;
  sum1=0.;
  for(nu=2;nu<=11;nu++)
    sum1+=(1-nu)*C[0][nu]*pow(chi,-nu);
  sum2=0.;
  for(nu=2;nu<=6;nu++)
    sum2+=(1-nu)*C[1][nu]*pow(chi,-nu);
  sum3=0.;
  for(nu=2;nu<=7;nu++)
    sum3+=(1-nu)*C[2][nu]*pow(chi,-nu);
  sum4=0.;
  for(nu=2;nu<=9;nu++)
    sum4+=(1-nu)*C[3][nu]*pow(chi,-nu);
  sum5=0.;
  for(nu=0;nu<=4;nu++)
    sum5+=C[6][nu]*pow(theta,-2-nu);
  return(
    -(C[0][1]+sum1+C[0][12]/chi)
    -(C[1][1]+sum2+C[1][ 7]/chi)*(theta-1.)
    -(C[2][1]+sum3+C[2][ 8]/chi)*pow(theta-1.,2)
    -(C[3][1]+sum4+C[3][10]/chi)*pow(theta-1.,3)
    +5.*C[4][1]*pow(chi,-6)*pow(theta,-23)*(theta-1.)
    -6.*pow(chi,5)*sum5
    );
  }

static double Epsilon3(double theta,double chi)
  {
  static double C[8][13]={ /* coefficients from Section 7.1.3, pg. 21; equation from Section 9.3, pg. 26 */
    {-6.839900000E+0,-1.722604200E-2,-7.771750390E+0, 4.204607520E+0,-2.768070380E+0, 2.104197070E+0,-1.146495880E+0,
      2.231380850E-1, 1.162503630E-1,-8.209005440E-2, 1.941292390E-2,-1.694705760E-3,-4.311577033E+0},
    {       0       , 7.086360850E-1, 1.236794550E+1,-1.203890040E+1, 5.404374220E+0,-9.938650430E-1, 6.275231820E-2,
     -7.747430160E+0,       0       ,       0       ,       0       ,       0       ,       0       },
    {       0       ,-4.298850920E+0, 4.314305380E+1,-1.416193130E+1, 4.041724590E+0, 1.555463260E+0,-1.665689350E+0,
      3.248811580E-1, 2.936553250E+1,       0       ,       0       ,       0       ,       0       },
    {       0       , 7.948418420E-6, 8.088597470E+1,-8.361533800E+1, 3.586365170E+1, 7.518959540E+0,-1.261606400E+1,
      1.097174620E+0, 2.121454920E+0,-5.465295660E-1, 8.328754130E+0,       0       ,       0       },
    { 2.759717760E-6,-5.090739850E-4,       0       ,       0       ,       0       ,       0       ,       0       ,
            0       ,       0       ,       0       ,       0       ,       0       ,       0       },
    { 2.106363320E+2,       0       ,       0       ,       0       ,       0       ,       0       ,       0       ,
            0       ,       0       ,       0       ,       0       ,       0       ,       0       },
    { 5.528935335E-2,-2.336365955E-1, 3.697071420E-1,-2.596415470E-1, 6.828087013E-2,       0       ,       0       ,
            0       ,       0       ,       0       ,       0       ,       0       ,       0       },
    {-2.571600553E+2,-1.518783715E+2, 2.220723208E+1,-1.802039570E+2, 2.357096220E+3,-1.462335698E+4, 4.542916630E+4,
     -7.053556432E+4, 4.381571428E+4,       0       ,       0       ,       0       ,       0       }};
  static double alpha0=0;
  int nu;
  double sum1,sum2,sum3,sum4,sum5,sum6,sum7,sum8,sum9;
  sum1=0.;
  for(nu=2;nu<=11;nu++)
    sum1+=nu*C[0][nu]*pow(chi,1-nu);
  sum2=0.;
  for(nu=2;nu<=6;nu++)
    sum2+=C[1][nu]*pow(chi,1-nu);
  sum3=0.;
  for(nu=2;nu<=6;nu++)
    sum3+=(nu-1)*C[1][nu]*pow(chi,1-nu);
  sum4=0.;
  for(nu=2;nu<=7;nu++)
    sum4+=C[2][nu]*pow(chi,1-nu);
  sum5=0.;
  for(nu=2;nu<=7;nu++)
    sum5+=(nu-2)*C[2][nu]*pow(chi,1-nu);
  sum6=0.;
  for(nu=2;nu<=9;nu++)
    sum6+=C[3][nu]*pow(chi,1-nu);
  sum7=0.;
  for(nu=2;nu<=9;nu++)
    sum7+=(nu-3)*C[3][nu]*pow(chi,1-nu);
  sum8=0.;
  for(nu=0;nu<=4;nu++)
    sum8+=(nu-3)*C[6][nu]*pow(theta,-2-nu);
  sum9=0.;
  for(nu=0;nu<=8;nu++)
    sum9+=C[7][nu]*(1.+nu*theta)*pow(theta-1.,nu);
  return(
    alpha0+C[0][0]-C[0][12]-C[5][0]-C[1][1]*chi+sum1-sum2+(C[0][12]-C[1][7])*log(chi)
   +(-C[1][7]-C[5][0]-(C[1][1]+2.*C[2][1])*chi+sum3-2*sum4-2.*C[2][8]*log(chi))*(theta-1.)
   +(-C[2][8]-(2.*C[2][1]+3.*C[3][1])*chi+sum5-3.*sum6-(C[2][8]+3.*C[3][10])*log(chi))*pow(theta-1.,2)
   +(-C[3][10]-3.*C[3][1]*chi+sum7-2.*C[3][10]*log(chi))*pow(theta-1.,3)
   +(23.*C[4][0]+28.*C[4][1]*pow(chi,-5))*pow(theta,-22)
   -(24.*C[4][0]+29.*C[4][1]*pow(chi,-5))*pow(theta,-23)
   +pow(chi,6)*sum8-sum9
   );
  }

static double Sigma3(double theta,double chi)
  {
  static double C[8][13]={ /* coefficients from Section 7.1.3, pg. 21; equation from Section 9.3, pg. 26 */
    {-6.839900000E+0,-1.722604200E-2,-7.771750390E+0, 4.204607520E+0,-2.768070380E+0, 2.104197070E+0,-1.146495880E+0,
      2.231380850E-1, 1.162503630E-1,-8.209005440E-2, 1.941292390E-2,-1.694705760E-3,-4.311577033E+0},
    {       0       , 7.086360850E-1, 1.236794550E+1,-1.203890040E+1, 5.404374220E+0,-9.938650430E-1, 6.275231820E-2,
     -7.747430160E+0,       0       ,       0       ,       0       ,       0       ,       0       },
    {       0       ,-4.298850920E+0, 4.314305380E+1,-1.416193130E+1, 4.041724590E+0, 1.555463260E+0,-1.665689350E+0,
      3.248811580E-1, 2.936553250E+1,       0       ,       0       ,       0       ,       0       },
    {       0       , 7.948418420E-6, 8.088597470E+1,-8.361533800E+1, 3.586365170E+1, 7.518959540E+0,-1.261606400E+1,
      1.097174620E+0, 2.121454920E+0,-5.465295660E-1, 8.328754130E+0,       0       ,       0       },
    { 2.759717760E-6,-5.090739850E-4,       0       ,       0       ,       0       ,       0       ,       0       ,
            0       ,       0       ,       0       ,       0       ,       0       ,       0       },
    { 2.106363320E+2,       0       ,       0       ,       0       ,       0       ,       0       ,       0       ,
            0       ,       0       ,       0       ,       0       ,       0       ,       0       },
    { 5.528935335E-2,-2.336365955E-1, 3.697071420E-1,-2.596415470E-1, 6.828087013E-2,       0       ,       0       ,
            0       ,       0       ,       0       ,       0       ,       0       ,       0       },
    {-2.571600553E+2,-1.518783715E+2, 2.220723208E+1,-1.802039570E+2, 2.357096220E+3,-1.462335698E+4, 4.542916630E+4,
     -7.053556432E+4, 4.381571428E+4,       0       ,       0       ,       0       ,       0       }};
  static double alpha1=0;
  int nu;
  double sum1,sum2,sum3,sum4,sum5;
  sum1=0.;
  for(nu=2;nu<=6;nu++)
    sum1+=C[1][nu]*pow(chi,1-nu);
  sum2=0.;
  for(nu=2;nu<=7;nu++)
    sum2+=C[2][nu]*pow(chi,1-nu);
  sum3=0.;
  for(nu=2;nu<=9;nu++)
    sum3+=C[3][nu]*pow(chi,1-nu);
  sum4=0.;
  for(nu=0;nu<=4;nu++)
    sum4+=(nu+2)*C[6][nu]*pow(theta,-3-nu);
  sum5=0.;
  for(nu=0;nu<=8;nu++)
    sum5+=(nu+1)*C[7][nu]*pow(theta-1.,nu);
  return(
    -alpha1
       -(C[1][1]*chi+sum1+C[1][ 7]*log(chi)+C[5][0])
    -2.*(C[2][1]*chi+sum2+C[2][ 8]*log(chi)        )*    (theta-1.  )
    -3.*(C[3][1]*chi+sum3+C[3][10]*log(chi)        )*pow(theta-1.,2)
    +(C[4][0]+C[4][1]*pow(chi,-5))*(22.*pow(theta,-23)-23.*pow(theta,-24))
    -C[5][0]*log(theta)
    +pow(chi,6)*sum4
    -sum5
    );
  }

static double Chi3(double theta,double beta)
  { /* iteratively solve for chi(theta,beta) in Region 3 */
  int iter;
  double chi,chi1,chi2;
  chi1=0.0207/0.0507785287;
  chi2=0.143 /0.0507785287;
  for(iter=0;iter<32;iter++)
    {
    chi=(chi1+chi2)/2.;
    if(Beta3(theta,chi)>beta)
      chi1=chi;
    else
      chi2=chi;
    }
  if(theta<0.992678)
    return(chi);
  chi1=chi;
  chi2=0.143 /0.0507785287;
  for(iter=0;iter<32;iter++)
    {
    chi=(chi1+chi2)/2.;
    if(Beta3(theta,chi)>beta)
      chi1=chi;
    else
      chi2=chi;
    }
  return(chi);
  }

static double Beta4(double theta,double chi)
  {
  static double D[3][5]={ /* coefficients from Section 7.1.4, pg. 22 */
    {-1.717616747E+0, 3.526389875E+0,-2.690899373E+0, 9.070982605E-1,-1.138791156E-1},
    { 1.301023613E+0,-2.642777743E+0, 1.996765362E+0,-6.661557013E-1, 8.270860589E-2},
    { 3.426663535E-4,-1.236521258E-3, 1.155018309E-3,       0       ,       0       }};
  static double theta1=9.626911787E-1; /* equation from Section 9.4, pg. 27 */
  int mu,nu;
  double beta3,sum1,sum2,sum3,y;
  beta3=Beta3(theta,chi);
  y=(1.-theta)/(1.-theta1);
  sum1=0.;
  for(mu=3;mu<=4;mu++)
    {
    sum2=0.;
    for(nu=0;nu<=4;nu++)
      sum2+=nu*D[mu-3][nu]*pow(y,mu)*pow(chi,-nu-1);
    sum1+=sum2;
    }
  sum3=0.;
  for(nu=0;nu<=2;nu++)
    sum3+=nu*D[2][nu]*pow(chi,nu-1);
  return(beta3+sum1-pow(y,32)*sum3);
  }

static double Epsilon4(double theta,double chi)
  {
  static double D[3][5]={ /* coefficients from Section 7.1.4, pg. 22 */
    {-1.717616747E+0, 3.526389875E+0,-2.690899373E+0, 9.070982605E-1,-1.138791156E-1},
    { 1.301023613E+0,-2.642777743E+0, 1.996765362E+0,-6.661557013E-1, 8.270860589E-2},
    { 3.426663535E-4,-1.236521258E-3, 1.155018309E-3,       0       ,       0       }};
  static double theta1=9.626911787E-1; /* equation from Section 9.4, pg. 27 */
  int mu,nu;
  double epsilon3,sum1,sum2,sum3,y;
  epsilon3=Epsilon3(theta,chi);
  y=(1.-theta)/(1.-theta1);
  sum1=0.;
  for(mu=3;mu<=4;mu++)
    {
    sum2=0.;
    for(nu=0;nu<=4;nu++)
      sum2+=((1-mu+nu)*y+mu/(1.-theta1))*D[mu-3][nu]*pow(y,mu-1)*pow(chi,-nu);
    sum1+=sum2;
    }
  sum3=0.;
  for(nu=0;nu<=2;nu++)
    sum3+=((31+nu)*y-32./(1.-theta1))*D[2][nu]*pow(chi,nu);
  return(epsilon3+sum1-pow(y,31)*sum3);
  }

static double Sigma4(double theta,double chi)
  {
  static double D[3][5]={ /* coefficients from Section 7.1.4, pg. 22 */
    {-1.717616747E+0, 3.526389875E+0,-2.690899373E+0, 9.070982605E-1,-1.138791156E-1},
    { 1.301023613E+0,-2.642777743E+0, 1.996765362E+0,-6.661557013E-1, 8.270860589E-2},
    { 3.426663535E-4,-1.236521258E-3, 1.155018309E-3,       0       ,       0       }};
  static double theta1=9.626911787E-1; /* equation from Section 9.4, pg. 27 */
  int mu,nu;
  double sigma3,sum1,sum2,sum3,y;
  sigma3=Sigma3(theta,chi);
  y=(1.-theta)/(1.-theta1);
  sum1=0.;
  for(mu=3;mu<=4;mu++)
    {
    sum2=0.;
    for(nu=0;nu<=4;nu++)
      sum2+=mu*D[mu-3][nu]*pow(y,mu-1)*pow(chi,-nu);
    sum1+=sum2;
    }
  sum3=0.;
  for(nu=0;nu<=2;nu++)
    sum3+=D[2][nu]*pow(chi,nu);
  return(sigma3+(sum1-32.*pow(y,31)*sum3)/(1.-theta1));
  }

static double Chi4(double theta,double beta)
  { /* iteratively solve for chi(theta,beta) in Region 4 */
  int iter;
  double chi,chi1,chi2;
  chi1=0.0207/0.0507785287;
  chi2=12.02*theta-10.94;
  for(iter=0;iter<32;iter++)
    {
    chi=(chi1+chi2)/2.;
    if(Beta4(theta,chi)>beta)
      chi1=chi;
    else
      chi2=chi;
    }
  return(chi);
  }

double Psat67(double Ts)/* saturation pressure [MPa] from temperature [K]*/
  {
  if(Ts<=273.16)
    return(0.000611244);
  if(Ts>=Tc)
    return(Pc);
  return(Pc*BetaK(Ts/Tc));
  }

double Tsat67(double Ps)/* saturation temperature [K] from pressure [MPa]*/
  {
  int iter;
  double T1,T2,Tsat;
  if(Ps<=0.000611244)
    return(273.16);
  if(Ps>=Pc)
    return(Tc);
  T1=273.16;
  T2=Tc;
  for(iter=0;iter<32;iter++)
    {
    Tsat=(T1+T2)/2.;
    if(Psat67(Tsat)<Ps)
      T1=Tsat;
    else
      T2=Tsat;
    }
  return(Tsat);
  }

enum{liquid,vapor,supercritical};

static void VHSofTP(double T,double P,double*V,double*H,double*S,int iopt)
  {
  /* This function takes temperature [K] and pressure [MPa] and returns
     the specific volume [cc/gm], enthalpy [kJ/kg], and entropy
     [kJ/kg/K]. The option (iopt) is used to obtain saturation
     properties. If iopt=liquid the saturated liquid state is returned
     and the pressure is ignored. If iopt=vapor the saturated vapor state is
     returned and the pressure is ignored. If iopt=other the state is
     determined from T and P. */
  static double theta1=9.626911787E-1;
  static double epsilonc=30.0536;
  static double sigmac=41.0133;
  int region;
  double beta,chi,epsilon,sigma,theta,Vc;
  theta=T/Tc;
  beta=P/Pc;
  if(iopt==liquid)
    {
    if(theta<theta1)
      {
      beta=BetaK(theta);
      chi=Chi1(theta,beta);
      sigma=Sigma1(theta,beta);
      epsilon=Epsilon1(theta,beta);
      region=1;
      }
    else if(theta<1.)
      {
      beta=BetaK(theta);
      chi=Chi4(theta,beta);
      sigma=Sigma4(theta,chi);
      epsilon=Epsilon4(theta,chi);
      region=4;
      }
    else
      {
      chi=1.;
      sigma=sigmac;
      epsilon=epsilonc;
      region=4;
      }
    }
  else if(iopt==vapor)
    {
    if(theta<theta1)
      {
      beta=BetaK(theta);
      chi=Chi2(theta,beta);
      sigma=Sigma2(theta,beta);
      epsilon=Epsilon2(theta,beta);
      region=2;
      }
    else if(theta<1.)
      {
      beta=BetaK(theta);
      chi=Chi3(theta,beta);
      sigma=Sigma3(theta,chi);
      epsilon=Epsilon3(theta,chi);
      region=3;
      }
    else
      {
      chi=1.;
      sigma=sigmac;
      epsilon=epsilonc;
      region=4;
      }
    }
  else
    {
    if(theta<theta1)
      {
      if(beta<BetaK(theta))
        region=2;
      else
        region=1;
      }
    else if(theta<1.)
      {
      if(beta<BetaL(theta))
        region=2;
      else if(beta<BetaK(theta))
        region=3;
      else
        region=4;
      }
    else
      {
      if(beta<BetaL(theta))
        region=2;
      else
        region=3;
      }
    if(region==1)
      {
      chi=Chi1(theta,beta);
      sigma=Sigma1(theta,beta);
      epsilon=Epsilon1(theta,beta);
      }
    else if(region==2)
      {
      chi=Chi2(theta,beta);
      sigma=Sigma2(theta,beta);
      epsilon=Epsilon2(theta,beta);
      }
    else if(region==3)
      {
      chi=Chi3(theta,beta);
      sigma=Sigma3(theta,chi);
      epsilon=Epsilon3(theta,chi);
      }
    else
      {
      chi=Chi4(theta,beta);
      if(fabs(Beta4(theta,chi)-beta)>0.00001)
        { /* If the Chi4() search fails switch to Region 3. */
        region=3;
        chi=Chi3(theta,beta);
        sigma=Sigma3(theta,chi);
        epsilon=Epsilon3(theta,chi);
        }
      else
        {
        sigma=Sigma4(theta,chi);
        epsilon=Epsilon4(theta,chi);
        }
      }
    }
  Vc=1./rhoc;
  V[0]=chi/Vc;
  S[0]=sigma*Pc*Vc/Tc;
  H[0]=epsilon*Pc*Vc;
  }

double V67(double T,double P,int iopt)
  {
  double H,S,V;
  VHSofTP(T,P,&V,&H,&S,iopt);
  return(10.*V);
  }

double H67(double T,double P,int iopt)
  {
  double H,S,V;
  VHSofTP(T,P,&V,&H,&S,iopt);
  return(H);
  }

double S67(double T,double P,int iopt)
  {
  double H,S,V;
  VHSofTP(T,P,&V,&H,&S,iopt);
  return(S);
  }

#ifdef _TEST

void Sat(double Ts)
  {
  double Hf,Hg,Ps,Sf,Sg,Vf,Vg;
  Ps=Psat67(Ts);
  Vf=V67(Ts,0.,liquid);
  Vg=V67(Ts,0.,vapor);
  Hf=H67(Ts,0.,liquid);
  Hg=H67(Ts,0.,vapor);
  Sf=S67(Ts,0.,liquid);
  Sg=S67(Ts,0.,vapor);
  printf("%7.3lf %12.9lf %7.5lf %12.5lf %6.1lf %6.1lf %6.4lf %6.4lf\n",Ts,Ps,Vf,Vg,Hf,Hg,Sf,Sg);
  }

void ListSat()
  {
  double T;
  printf("  Ts          Ps        Vf       Vg         Hf     Hg     Sf     Sg\n");
  Sat(273.16);
  for(T=275.;T<645.1;T+=5.)
    Sat(T);
  Sat(Tc);
  }

void ListSup()
  {
  double H,P,S,T,V;
  printf("  T     P        V         H      S\n");
  for(T=275.;T<1601.;T+=25.)
    {
    for(P=0.01;P<175.;P*=2.)
      {
      V=V67(T,P,supercritical);
      H=H67(T,P,supercritical);
      S=S67(T,P,supercritical);
      printf("%4.0lf %6.2lf %11.5lf %6.1lf %7.4lf\n",T,P,V,H,S);
      }
    }
  }

int main(int argc,char**argv,char**envp)
  {
  printf("1967 Steam Properties (Industrial Formulation)\n");
  printf("developed by Meyer, McClintock, Silvestri, and Spencer\n");
  printf("implemented by Dudley J. Benton\n");
  ListSat();
  ListSup();
  return(0);
  }

#endif
