/* 1969 Steam Properties (Scientific Formulation)
   developed by Keenan, Keyes, Hill, and Moore
   implemented by Dudley J. Benton */

#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <float.h>
#define _USE_MATH_DEFINES
#include <math.h>

#define E 4.8                  /* unitless parameter */
#define Pc 22.0894             /* critical pressure [MPa] */
#define R 0.461504339          /* gas constant [J/gm/K] */
#define rhoc 0.316819637748426 /* critical density [gm/cm^3] */
#define Tc 647.245             /* critical temperature [K] */

static double A[70]={
   29.492937,-5.1985860, 6.8335354,-0.1564104 ,-6.3972405,-3.9661401,-0.69048554,
  -132.13917, 7.7779182,-26.149751,-0.72546108, 26.409282, 15.453061, 2.7407416 ,
   274.64632,-33.301902, 65.326396,-9.2734289 ,-47.740374,-29.142470,-5.1028070 ,
  -360.93828,-16.254622,-26.181978, 4.3125840 , 56.323130, 29.568796, 3.9636085 ,
   342.18431,-177.31074, 0.       , 0.        , 0.       , 0.       , 0.        ,
  -244.50042, 127.48742, 0.       , 0.        , 0.       , 0.       , 0.        ,
   155.18535, 137.46153, 0.       , 0.        , 0.       , 0.       , 0.        ,
   5.9728487, 155.97836, 0.       , 0.        , 0.       , 0.       , 0.        ,
  -410.30848, 337.31180,-137.46618, 6.7874983 , 136.87317, 79.847970, 13.041253 ,
  -416.05860,-209.88866,-733.96848, 10.401717 , 645.81880, 399.17570, 71.531353 };

static double C[8]={1857.065,3229.12,-419.465,36.6649,-20.5516,4.85233,46.,-1011.249};

static double F[8]={-741.9242,-29.721,-11.55286,-0.8685635,0.1094098,0.439993,0.2520658,0.05218684};

static double fA(double T) /* zero density Helmholtz free energy a=u-Ts */
  {
  int i;
  double a,tau,tt;
  tau=1000./T;
  tt=1.;
  for(a=i=0;i<6;i++)
    {
    a+=C[i]/tt;
    tt*=tau;
    }
  a+=(C[6]+C[7]/tau)*log(T);
  return(a);
  }

double KKHMf(double Tr)
  {
  return(fA(Tr*Tc)/R/Tc);
  }

static double fAt(double T) /* derivative of zero density HFE with respect to tau */
  {
  int i;
  double at,tau,tt;
  tau=1000./T;
  at=0.;
  tt=tau;
  for(i=1;i<6;i++)
    {
    tt*=tau;
    at-=i*C[i]/tt;
    }
  at-=(C[6]+C[7]*(1.+log(T))/tau)/tau;
  return(at);
  }

static void tAt(double T,double*a,double*at) /* previous two functions combined */
  {
  int i;
  double lnT,tau,tt;
  tau=1000./T;
  tt=1.;
  for(a[0]=at[0]=i=0;i<6;i++)
    {
    a[0]+=C[i]/tt;
    at[0]-=i*C[i]/tt/tau;
    tt*=tau;
    }
  lnT=log(T);
  a[0]+=(C[6]+C[7]/tau)*lnT;
  at[0]-=(C[6]+C[7]*(1.+lnT)/tau)/tau;
  }

static double fQ(double tau,double rho)
  {
  int i,j;
  double dt,q,r,rhoa,s,tauc;
  tauc=1000./Tc;
  dt=1.;
  rhoa=0.634;
  for(q=j=0;j<7;j++)
    {
    r=1.;
    for(s=i=0;i<8;i++)
      {
      if(j>1&&i>3)
        break;
      s+=A[7*i+j]*r;
      r*=rho-rhoa;
      }
    s+=exp(-E*rho)*(A[7*8+j]+A[7*9+j]*rho);
    q+=dt*s;
    if(j==0)
      dt=tau-tauc;
    else
      dt*=tau-2.5;
    rhoa=1.;
    }
  return(q);
  }

double KKHMq(double Tr,double r)
  {
  return(fQ(1000./(Tr*Tc),r*rhoc));
  }

static double fQt(double tau,double rho) /* dQ/dtau */
  {
  int i,j;
  double ct,dt,qt,r,rhoa,s,tauc;
  tauc=1000./Tc;
  dt=ct=rhoa=1.;
  qt=0.;
  for(j=1;j<7;j++)
    {
    r=1.;
    for(s=i=0;i<8;i++)
      {
      if(j>1&&i>3)
        break;
      s+=A[7*i+j]*r;
      r*=rho-rhoa;
      }
    s+=exp(-E*rho)*(A[7*8+j]+A[7*9+j]*rho);
    qt+=ct*dt*s;
    if(j==1)
      ct=2.*tau-tauc-2.5;
    else
      {
      ct+=tau-tauc;
      dt*=tau-2.5;
      }
    }
  return(qt);
  }

static double fQr(double tau,double rho) /* dQ/drho */
  {
  int i,j;
  double dt,qr,r,rhoa,s,tauc;
  tauc=1000./Tc;
  dt=1.;
  rhoa=0.634;
  for(qr=j=0;j<7;j++)
    {
    r=1.;
    s=0.;
    for(i=1;i<8;i++)
      {
      if(j>1&&i>3)
        break;
      s+=A[7*i+j]*i*r;
      r*=rho-rhoa;
      }
    s+=exp(-E*rho)*(-A[7*8+j]*E+A[7*9+j]*(1.-E*rho));
    qr+=dt*s;
    if(j==0)
      dt=tau-tauc;
    else
      dt*=tau-2.5;
    rhoa=1.;
    }
  return(qr);
  }

double P69(double T,double rho)
  {
  double tau;
  tau=1000./T;
  return(((fQr(tau,rho)*rho+fQ(tau,rho))*rho+1.)*rho*R*T);
  }

double H69(double T,double rho)
  {
  double a,at,Q,Qr,Qt,tau;
  tau=1000./T;
  Q=fQ(tau,rho);
  Qr=fQr(tau,rho);
  Qt=fQt(tau,rho);
  a=fA(T);
  at=fAt(T);
  return(R*T*(rho*tau*Qt+1.+rho*Q+rho*rho*Qr)+a+tau*at);
  }

double S69(double T,double rho)
  {
  double at,Q,Qt,tau;
  tau=1000./T;
  Q=fQ(tau,rho);
  Qt=fQt(tau,rho);
  at=fAt(T);
  return(-R*(log(rho)+rho*(Q-tau*Qt))+at*tau/T);
  }

double Psat69(double Ts)  /* saturation pressure [MPa] from temperature [K]*/
  {
  int i;
  double f,tau,x;
  if(Ts<=273.16)
    return(0.000612049);
  if(Ts>=Tc)
    return(Pc);
  tau=1000./Ts;
  x=3.3815-0.01*Ts;
  f=F[7];
  for(i=6;i>=0;i--)
    f=f*x+F[i];
  return(Pc*exp(1E-5*tau*(Tc-Ts)*f));
  }

double Tsat69(double Ps) /* saturation temperature [K] from pressure [MPa] */
  {
  int i;
  double T1,T2,Ts;
  if(Ps<=0.000612049)
    return(273.16);
  if(Ps>=Pc)
    return(Tc);
  T1=273.16;
  T2=Tc;
  for(i=0;i<32;i++)
    {
    Ts=(T1+T2)/2.;
    if(Psat69(Ts)<Ps)
      T1=Ts;
    else
      T2=Ts;
    }
  return(Ts);
  }

enum{liquid,vapor,supercritical};

static double rhof(double T)
  {
  int i;
  double P1,P2,P3,P,r1,r2,r3,Tr,x;
  if(T>=Tc)
    return(rhoc);
  Tr=T/Tc;
  x=pow(1.-Tr,1./3.);
  P=Psat69(T);
  x=pow(647.286-T,0.333);
  r1=rhoc+((-0.000938708*x+0.0112886)*x+0.0643931)*x;
  r2=r1*1.01;
  P1=P69(T,r1);
  P2=P69(T,r2);
  for(i=0;i<32;i++)
    {
    if(fabs(P2-P1)<P*FLT_EPSILON)
      break;
    r3=(r1*(P2-P)-r2*(P1-P))/(P2-P1);
    P3=P69(T,r3);
    if(fabs(P3/P-1.)<sqrt(FLT_EPSILON))
      break;
    if(fabs(P1-P)<fabs(P2-P))
      {
      r2=r3;
      P2=P3;
      }
    else
      {
      r1=r3;
      P1=P3;
      }
    }
  return(r3);
  }

static double rhog(double T)
  {
  int i;
  double P1,P2,P3,P,r1,r2,r3,x,Z;
  if(T>=Tc)
    return(rhoc);
  P=Psat69(T);
  x=pow(Tc-T,0.4648);
  Z=0.233396+((-0.000215689*x+0.00313438)*x+0.0525888)*x;
  r1=P/Z/R/T;
  r2=r1/1.1;
  P1=P69(T,r1);
  P2=P69(T,r2);
  for(i=0;i<32;i++)
    {
    if(fabs(P2-P1)<P*FLT_EPSILON)
      break;
    r3=(r1*(P2-P)-r2*(P1-P))/(P2-P1);
    P3=P69(T,r3);
    if(fabs(P3/P-1.)<sqrt(FLT_EPSILON))
      break;
    if(fabs(P1-P)<fabs(P2-P))
      {
      r2=r3;
      P2=P3;
      }
    else
      {
      r1=r3;
      P1=P3;
      }
    }
  return(r3);
  }

double R69(double T,double P,int region)
  {
  int i;
  double P1,P2,P3,Ps,r1,r2,r3,r4,rf,rg,Z;
  if(region==liquid||region==vapor)
    {
    if(T>=Tc)
      return(rhoc);
    if(region==liquid)
      return(rhof(T));
    return(rhog(T));
    }
  if(T<Tc)
    {
    Ps=Psat69(T);
    if(P>Ps)
      {
      region=liquid;
      rf=r1=rhof(T);
      r2=r1*1.01;
      }
    else
      {
      region=vapor;
      rg=r1=rhog(T);
      r2=r1/1.1;
      }
    }
  else
    {
    Z=2.5;
    r1=rhoc*3.;
    r2=P/Z/R/T;
    }
  P1=P69(T,r1);
  P2=P69(T,r2);
  r4=(r1+r2)/2.;
  for(i=0;i<64;i++)
    {
    if(fabs(P2-P1)<P*FLT_EPSILON)
      break;
    r3=(r1*(P2-P)-r2*(P1-P))/(P2-P1);
    if(i>=32)
      r3=(r3+r4)/2.;
    if(region==liquid)
      if(r3<rf)
        r3=rf;
    if(region==vapor)
      if(r3>rg)
        r3=rg;
    if(r3<(r1+r2)/4.)
      r3=(r1+r2)/4.;
    P3=P69(T,r3);
    if(fabs(P3/P-1.)<sqrt(FLT_EPSILON))
      break;
    if(fabs(P1-P)<fabs(P2-P))
      {
      r2=r3;
      P2=P3;
      }
    else
      {
      r1=r3;
      P1=P3;
      }
    r4=r3;
    }
  return(r3);
  }

#ifdef _TEST
#define PPP P69
#define HHH H69
#define SSS S69
#define RHO R69
#define Psat Psat69

void Sat(double Ts)
  {
  double Hf,Hg,Ps,rf,rg,Sf,Sg;
  Ps=Psat(Ts);
  rf=RHO(Ts,0.,liquid);
  rg=RHO(Ts,0.,vapor);
  Hf=HHH(Ts,rf);
  Hg=HHH(Ts,rg);
  Sf=SSS(Ts,rf);
  Sg=SSS(Ts,rg);
  printf("%7.3lf %12.9lf %7.5lf %12.5lf %6.1lf %6.1lf %6.4lf %6.4lf\n",Ts,Ps,1./rf,1./rg,Hf,Hg,Sf,Sg);
  }

void ListSat()
  {
  double T;
  printf("  Ts          Ps        Vf       Vg         Hf     Hg     Sf     Sg\n");
  Sat(273.16);
  for(T=275.;T<645.1;T+=5.)
    Sat(T);
  Sat(Tc);
  }

void ListSup()
  {
  double H,P,r,S,T;
  printf("  T     P        V         H      S\n");
  for(T=275.;T<1601.;T+=25.)
    {
    for(P=0.01;P<175.;P*=2.)
      {
      r=RHO(T,P,supercritical);
      H=HHH(T,r);
      S=SSS(T,r);
      printf("%4.0lf %6.2lf %11.5lf %6.1lf %7.4lf\n",T,P,1./r,H,S);
      }
    }
  }

void ZTr_vs_rho(char*name)
  {
  double TT[]={275,300,325,350,375,400,425,450,475,500,525,550,575,600,
    625,Tc,675,700,725,750,775,800,825,850,875,900,925,950,975,1000};
  int i,n=sizeof(TT)/sizeof(TT[0]);
  double Pr,rho,rmx,Tr,Vc,Vr,Z,Zc;
  FILE*fp;
  fp=fopen(name,"wt");
  fprintf(fp,"Vc/V");
  for(i=0;i<n;i++)
    fprintf(fp,",%lG",TT[i]/Tc);
  fprintf(fp,"\n");
  Vc=1./rhoc;
  Zc=Pc*Vc/R/Tc;
  rmx=R69(TT[n-1],200.,supercritical);
  for(rho=rmx;rho>0.01;rho*=0.999)
    {
    fprintf(fp,"%lG",rho/rhoc);
    for(i=0;i<n;i++)
      {
      Vr=rhoc/rho;
      Pr=P69(TT[i],rho)/Pc;
      Tr=TT[i]/Tc;
      Z=Zc*Pr*Vr/Tr;
      fprintf(fp,",%lG",Z*Tr);
      }
    fprintf(fp,"\n");
    }
  fclose(fp);
  }

int main(int argc,char**argv,char**envp)
  {
  printf("1969 Steam Properties (Scientific Formulation)\n");
  printf("developed by Keenan, Keyes, Hill, and Moore\n");
  printf("implemented by Dudley J. Benton\n");
  ListSat();
  ListSup();
  ZTr_vs_rho("KKHM_ZTr_vs_rho.csv");
  return(0);
  }
#endif
