/* NBS/NRC 1984 Steam Properties (Scientific Formulation)
   developed by Haar, Gallagher, and Kell
   implemented by Dudley J. Benton */

#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#define _USE_MATH_DEFINES
#include <math.h>

#define rhoc 0.322       /* gm/cc */
#define Tc   647.126     /* K */
#define Pc   22.055      /* MPa */
#define Uref 4321.900017 /* kJ/kg */
#define Sref 7.640203087 /* kJ/kg/K */

#define alpha 11.
#define beta  (133./3.)
#define gamma (7./2.)
#define Po    0.101325
#define R     0.46152
#define To    647.073

static double BP[6]={0.7478629,-0.3540782,0.,0.007159876,0.,-0.003528426};
static double BQ[6]={1.1278334,-0.5944001,-5.010996,0.,0.63684256};
static double AA[4]={34.,40.,30.,1053.};
static double BE[4]={20000.,20000.,40000.,25.};
static double RR[4]={0.319,0.319,0.319,1.55};
static double TT[4]={640.,640.,641.6,270.};
static double CC[18]={        1.973027E+01, 2.096627E+01,-4.834295E-01,
                6.057432E+00, 2.256024E+01,-9.875324E+00,-4.313554E+00,
                4.581558E-01,-4.775490E-02, 4.123846E-03,-2.792905E-04,
                1.448169E-05,-5.647366E-07, 1.620045E-08,-3.303823E-10,
                4.519161E-12,-3.707341E-14, 1.375461E-16};
static double GG[40]={       -5.306297E+02, 2.274490E+03, 7.877933E+02,
               -6.983053E+01, 1.786383E+04,-3.951473E+04, 3.380388E+04,
               -1.385505E+04,-2.563744E+05, 4.821258E+05,-3.418302E+05,
                1.222316E+05, 1.179743E+06,-2.173481E+06, 1.082995E+06,
               -2.544200E+05,-3.137778E+06, 5.291191E+06,-1.380258E+06,
               -2.510991E+05, 4.656183E+06,-7.275278E+06, 4.177425E+05,
                1.401636E+06,-3.155523E+06, 4.792967E+06, 4.091267E+05,
               -1.362637E+06, 6.962522E+05,-1.083490E+06,-2.272283E+05,
                3.836549E+05, 6.883326E+03, 2.175725E+04,-2.662794E+03,
               -7.073042E+04,-2.250000E-01,-1.680000E+00, 5.500000E-02,
               -9.300000E+01};
static int kk[40]={1,1,1,1,2,2,2,2,3,3,3,3,4,4,4,4,5,5,5,5,
                   6,6,6,6,7,7,7,7,9,9,9,9,3,3,1,5,2,2,2,4};
static int ll[40]={1,2,4,6,1,2,4,6,1,2,4,6,1,2,4,6,1,2,4,6,
                   1,2,4,6,1,2,4,6,1,2,4,6,0,3,3,3,0,2,0,0};

static double bb(double T)
  {
  return(BP[0]+BP[1]*(log(T/To))+BP[3]*pow(To/T,3)+BP[5]*pow(To/T,5));
  }

static double dbdT(double T)
  {
  return(BP[1]/T-(3.*BP[3]*pow(To/T,4)+5.*BP[5]*pow(To/T,6))/To);
  }

static double BB(double T)
  {
  return(BQ[0]+BQ[1]*(To/T)+BQ[2]*pow(To/T,2)+BQ[4]*pow(To/T,4));
  }

static double dBdT(double T)
  {
  return(-(BQ[1]*pow(To/T,2)+2.*BQ[2]*pow(To/T,3)+4.*BQ[4]*pow(To/T,5))/To);
  }

static double A84(double T,double rho)
  {
  int i;
  double Abas,Agas,Ares,b,B,delta,E,tau,y;
  b=bb(T);
  B=BB(T);
  y=b*rho/4.;
  Abas=(-log(1.-y)-(beta-1.)/(1.-y)
        +(alpha+beta+1.)/2./pow(1.-y,2)
        +4.*y*(B/b-gamma)
        -(alpha-beta+3.)/2.
        +log(rho*R*T/Po))*R*T;
  E=exp(-rho);
  for(Ares=i=0;i<36;i++)
    Ares+=GG[i]/kk[i]*pow(To/T,ll[i])*pow(1.-E,kk[i]);
  for(i=36;i<40;i++)
    {
    delta=rho/RR[i-36]-1.;
    tau=T/TT[i-36]-1.;
    Ares+=GG[i]*pow(delta,ll[i])
         *exp(-AA[i-36]*pow(delta,kk[i])-BE[i-36]*tau*tau);
    }
  Agas=1.+(CC[0]/(T/100.)+CC[1])*log(T/100.);
  for(i=2;i<18;i++)
    Agas+=CC[i]*pow(T/100.,i-5);
  Agas*=-R*T;
  return(Abas+Ares+Agas+R*(Uref+Sref*T));
  }

double S84(double T,double rho)
  {
  int i;
  double b,B,delta,E,Sbas,Sgas,Sres,tau,Tdb,TdB,x,y;
  Sgas=1.+CC[0]/(T/100.)+CC[1]*(1.+log(T/100.));
  for(i=2;i<18;i++)
    Sgas+=CC[i]*(i-4)*pow(T/100.,i-5);
  Sgas*=R;
  b=bb(T);
  B=BB(T);
  Tdb=T*dbdT(T);
  TdB=T*dBdT(T);
  y=b*rho/4.;
  x=1.-y;
  Sbas=log(x)+(beta-1.)/x-(alpha+beta+1.)/2./x/x-rho*(b+Tdb)*(B/b-gamma)
      +(alpha-beta+1.)/2.-log(rho*R*T/Po)-Tdb*rho/4./x+(beta-1.)*Tdb*rho/4./x/x
      -(alpha+beta+1.)*Tdb*rho/4./x/x/x-rho*(TdB-B*Tdb/b);
  Sbas*=R;
  E=exp(-rho);
  for(Sres=i=0;i<36;i++)
    Sres+=GG[i]*ll[i]*pow(To/T,ll[i]+1)*pow(1.-E,kk[i])/kk[i]/To;
  for(i=36;i<40;i++)
    {
    delta=rho/RR[i-36]-1.;
    tau=T/TT[i-36]-1.;
    Sres-=2.*GG[i]*pow(delta,ll[i])*BE[i-36]*tau
      *exp(-AA[i-36]*pow(delta,kk[i])-BE[i-36]*tau*tau)/TT[i-36];
    }
  return(Sbas+Sres+Sgas-R*Sref);
  }

static double U84(double T,double rho)
  {
  return(A84(T,rho)+T*S84(T,rho));
  }

double P84(double T,double rho)
  {
  int i;
  double b,B,delta,E,q,Q,tau,y;
  E=exp(-rho);
  for(Q=i=0;i<36;i++)
    Q+=GG[i]*E*pow(1.-E,kk[i]-1)*pow(To/T,ll[i]);
  for(i=36;i<40;i++)
    {
    delta=rho/RR[i-36]-1.;
    tau=T/TT[i-36]-1.;
    Q+=GG[i]*(ll[i]-kk[i]*AA[i-36]*pow(delta,kk[i]))*pow(delta,ll[i]-1)
      *exp(-AA[i-36]*pow(delta,kk[i])-BE[i-36]*tau*tau)/RR[i-36];
    }
  b=bb(T);
  B=BB(T);
  y=b*rho/4.;
  q=(1.+(alpha+beta*y)*y)/pow(1.-y,3)+4.*y*(B/b-gamma);
  return(rho*(q*R*T+rho*Q));
  }

static double GGG(double T,double rho)
  {
  return(A84(T,rho)+P84(T,rho)/rho);
  }

double H84(double T,double rho)
  {
  return(A84(T,rho)+T*S84(T,rho)+P84(T,rho)/rho);
  }

double Psat84(double Ts)
  {
  static double A[8]={-7.8889166,2.5514255,-6.716169,33.239495,
                    -105.38479,174.35319,-148.39348,48.631602};
  double H,S,V,W;
  int i;
  if(Ts<=273.16)
    return(0.000610716);
  if(Ts>=Tc)
    return(Pc);
  V=Ts/647.25;
  W=fabs(1.-V);
  S=sqrt(W);
  H=A[7];
  for(i=6;i>=0;i--)
    H=H*S+A[i];
  return(Pc*exp(H*W/V));
  }

double Tsat84(double Ps)
  {
  int i;
  double T1,T2,Ts;
  if(Ps<=0.000610716)
    return(273.16);
  if(Ps>=Pc)
    return(Tc);
  T1=273.16;
  T2=Tc;
  for(i=0;i<32;i++)
    {
    Ts=(T1+T2)/2.;
    if(Psat84(Ts)<Ps)
      T1=Ts;
    else
      T2=Ts;
    }
  return(Ts);
  }

enum{liquid,vapor,supercritical};

static double RHOF(double T)
  {
  int i;
  double P1,P2,P3,P,r1,r2,r3,Tr,x;
  if(T>=Tc)
    return(rhoc);
  Tr=T/Tc;
  x=pow(1.-Tr,1./3.);
  P=Psat84(T);
  r1=(((((-30.510632553*x+82.6149432834)*x-88.382040957)*x
    +46.2620966218)*x-11.4769902281)*x+1.81999672256)*x+0.322;
  r2=r1*1.01;
  P1=P84(T,r1);
  P2=P84(T,r2);
  for(i=0;i<32;i++)
    {
    if(fabs(P2-P1)<P*FLT_EPSILON)
      break;
    r3=(r1*(P2-P)-r2*(P1-P))/(P2-P1);
    P3=P84(T,r3);
    if(fabs(P3/P-1.)<sqrt(FLT_EPSILON))
      break;
    if(fabs(P1-P)<fabs(P2-P))
      {
      r2=r3;
      P2=P3;
      }
    else
      {
      r1=r3;
      P1=P3;
      }
    }
  return(r3);
  }

static double RHOG(double T)
  {
  int i;
  double P1,P2,P3,P,r1,r2,r3,Tr,x,Z;
  if(T>=Tc)
    return(rhoc);
  Tr=T/Tc;
  x=pow(1.-Tr,1./3.);
  P=Psat84(T);
  Z=((-2.6054*Tr+3.5799)*Tr-1.4087)*Tr+0.66743+0.58211*pow(1.-Tr,1./3.);
  r1=P/Z/R/T;
  r2=r1/1.1;
  P1=P84(T,r1);
  P2=P84(T,r2);
  for(i=0;i<32;i++)
    {
    if(fabs(P2-P1)<P*FLT_EPSILON)
      break;
    r3=(r1*(P2-P)-r2*(P1-P))/(P2-P1);
    P3=P84(T,r3);
    if(fabs(P3/P-1.)<sqrt(FLT_EPSILON))
      break;
    if(fabs(P1-P)<fabs(P2-P))
      {
      r2=r3;
      P2=P3;
      }
    else
      {
      r1=r3;
      P1=P3;
      }
    }
  return(r3);
  }

double R84(double T,double P,int region)
  {
  int i;
  double P1,P2,P3,Pr,Ps,r1,r2,r3,rf,rg,Tr,Z;
  if(region==liquid||region==vapor)
    {
    if(T>=Tc)
      return(rhoc);
    if(region==liquid)
      return(RHOF(T));
    return(RHOG(T));
    }
  if(T<Tc)
    {
    Ps=Psat84(T);
    if(P>Ps)
      {
      region=liquid;
      rf=r1=RHOF(T);
      r2=r1*1.01;
      }
    else
      {
      region=vapor;
      rg=r1=RHOG(T);
      r2=r1/1.1;
      }
    }
  else
    {
    Tr=T/Tc;
    Pr=P/Pc;
    Z=0.233+(1.-0.233)*(Tr-1.)/1.5;
    Z=1.-Z+2.*Z*Pr/(1.+Pr*Pr);
    r1=P/Z/R/T;
    r2=1.1*r1;
    }
  P1=P84(T,r1);
  P2=P84(T,r2);
  for(i=0;i<32;i++)
    {
    if(fabs(P2-P1)<P*FLT_EPSILON)
      break;
    r3=(r1*(P2-P)-r2*(P1-P))/(P2-P1);
    if(region==liquid)
      if(r3<rf)
        r3=rf;
    if(region==vapor)
      if(r3>rg)
        r3=rg;
    if(r3<(r1+r2)/4.)
      r3=(r1+r2)/4.;
    P3=P84(T,r3);
    if(fabs(P3/P-1.)<sqrt(FLT_EPSILON))
      break;
    if(fabs(P1-P)<fabs(P2-P))
      {
      r2=r3;
      P2=P3;
      }
    else
      {
      r1=r3;
      P1=P3;
      }
    }
  return(r3);
  }

#ifdef _TEST

#define PPP P84
#define HHH H84
#define SSS S84
#define RHO R84
#define Psat Psat84

void Sat(double Ts)
  {
  double Hf,Hg,Ps,rf,rg,Sf,Sg;
  Ps=Psat(Ts);
  rf=RHO(Ts,0.,liquid);
  rg=RHO(Ts,0.,vapor);
  Hf=HHH(Ts,rf);
  Hg=HHH(Ts,rg);
  Sf=SSS(Ts,rf);
  Sg=SSS(Ts,rg);
  printf("%7.3lf %12.9lf %7.5lf %12.5lf %6.1lf %6.1lf %6.4lf %6.4lf\n",Ts,Ps,1./rf,1./rg,Hf,Hg,Sf,Sg);
  }

void ListSat()
  {
  double T;
  printf("  Ts          Ps        Vf       Vg         Hf     Hg     Sf     Sg\n");
  Sat(273.16);
  for(T=275.;T<645.1;T+=5.)
    Sat(T);
  Sat(Tc);
  }

void ListSup()
  {
  double H,P,r,S,T;
  printf("  T     P        V         H      S\n");
  for(T=275.;T<1601.;T+=25.)
    {
    for(P=0.01;P<175.;P*=2.)
      {
      r=RHO(T,P,supercritical);
      H=HHH(T,r);
      S=SSS(T,r);
      printf("%4.0lf %6.2lf %11.5lf %6.1lf %7.4lf\n",T,P,1./r,H,S);
      }
    }
  }

void ZTr_vs_rho(char*name)
  {
  double TT[]={275,300,325,350,375,400,425,450,475,500,525,550,575,600,
    625,Tc,675,700,725,750,775,800,825,850,875,900,925,950,975,1000};
  int i,n=sizeof(TT)/sizeof(TT[0]);
  double Pr,rho,rmx,Tr,Vc,Vr,Z,Zc;
  FILE*fp;
  fp=fopen(name,"wt");
  fprintf(fp,"Vc/V");
  for(i=0;i<n;i++)
    fprintf(fp,",%lG",TT[i]/Tc);
  fprintf(fp,"\n");
  Vc=1./rhoc;
  Zc=Pc*Vc/R/Tc;
  rmx=R84(TT[n-1],200.,supercritical);
  for(rho=rmx;rho>0.01;rho*=0.999)
    {
    fprintf(fp,"%lG",rho/rhoc);
    for(i=0;i<n;i++)
      {
      Vr=rhoc/rho;
      Pr=P84(TT[i],rho)/Pc;
      Tr=TT[i]/Tc;
      Z=Zc*Pr*Vr/Tr;
      fprintf(fp,",%lG",Z*Tr);
      }
    fprintf(fp,"\n");
    }
  fclose(fp);
  }

int main(int argc,char**argv,char**envp)
  {
  printf("NBS/NRC 1984 Steam Properties (Scientific Formulation)\n");
  printf("developed by Haar, Gallagher, and Kell\n");
  printf("implemented by Dudley J. Benton\n");
  ListSat();
  ListSup();
  ZTr_vs_rho("NBSNRC_ZTr_vs_rho.csv");
  return(0);
  }

#endif
