/* simple single-pressure HRSG and
   Combined Cycle Power Plant Model
   using SI units
   developed by Dudley J. Benton */

#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#define _USE_MATH_DEFINES
#include <math.h>
#include "..\\AllSteam\\AllSteam.h"

static double fmin(double x,double y)
  {
  if(x<y)
    return(x);
  return(y);
  }

static double fmax(double x,double y)
  {
  if(x>y)
    return(x);
  return(y);
  }

double LMTD(double dT1,double dT2)
  {
  if(dT1<=0.||dT2<=0.)
    return(0.);
  if(fabs(dT1-dT2)<0.001)
    return(sqrt(dT1*dT2));
  return((dT1-dT2)/log(dT1/dT2));
  }

/*******************************************************************************
  GAS TURBINE DATA AND FUNCTIONS
*******************************************************************************/

typedef struct{double kWnet,kJkWh,EGW,EGT;}GT;

GT LM6000={46720,  /* net power output [kW] */
            8757,  /* net heat rate [kJ/kWh] */
             300,  /* exhaust flow [kg/s] */
             730}; /* exhaust temperature [deg K] */

GT GasTurbinePerformance(double P,double T,double RH)
  {
  static GT gt;
  gt.kWnet=LM6000.kWnet    *(P/0.101325)*(-0.00433*(T-286.)+1.)*( 0.00655*(RH-0.6)+1.);
  gt.kJkWh=LM6000.kJkWh    *(0.101325/P)*( 0.00127*(T-286.)+1.)*( 0.00153*(RH-0.6)+1.);
  gt.EGW  =LM6000.EGW      *(P/0.101325)*(-0.00310*(T-286.)+1.)*(-0.00533*(RH-0.6)+1.);
  gt.EGT  =LM6000.EGT+132.3*(P-0.101325)+  0.489  *(T-286.)    +  2.39   *(RH-0.6)    ;
  return(gt);
  }

/*******************************************************************************
  STEAM TURBINE DATA AND FUNCTIONS
*******************************************************************************/

int year=-97; /* steam properties (must be 67, 69, 84, 95, or 97) */
              /* use +year for English and -year for SI units */

typedef struct{double T,P,H;}TPH;

void Turbine(double P1,double H1,double P2,double efficiency,double*H2)
  {
  double H2S,S1;
  S1=SofPH(P1,H1,year);
  H2S=HofPS(P2,S1,year);
  *H2=H1+(H2S-H1)*efficiency;
  }

void Pump(double P1,double H1,double P2,double efficiency,double*H2)
  {
  double H2S,S1;
  S1=SofPH(P1,H1,year);
  H2S=HofPS(P2,S1,year);
  *H2=H1+(H2S-H1)/efficiency;
  }

struct{double swc,eff;}stg={
  1.50,  /* swallowing capacity [MPa/kg/s] */
  0.90}; /* isentropic efficiency */

struct{double eff;}bfp={0.85};
struct{double eff;}gen={0.90};

/*******************************************************************************
  CONDENSER AND HEAT EXCHANGER PERFORMANCE FUNCTIONS
*******************************************************************************/

double Tcond(double kWt,double Tin)
  {
  return(Tin+kWt/1000.);
  }

double Cp=1.12;  /* GT exhaust gas specific heat [kJ/kg/C] */
double UAta=200; /* target UA [kW/C] */
double Tstm=700; /* main steam temperature [K] */

/*******************************************************************************
  COMBINED CYCLE DATA AND FUNCTIONS
*******************************************************************************/

void CCPP(double baro,double Tamb,double RH,double Tccw)
  {
  int iter;
  double dTec,dTev,dTsh,Fstm,Qec,Qev,Qfu,Qsh,T1,T2,T3,T4,UAec,UAev,UAsh,UAto;
  GT gt;
  TPH bfw,cnd,eco,evp,mst,uep;
  struct{double cnd,net,pmp,stg;}kW;
  gt=GasTurbinePerformance(baro,Tamb,RH);
  Qfu=gt.kWnet*gt.kJkWh/3600.;
  mst.T=Tstm;
  Fstm=gt.EGW/50.;
  uep.T=Tccw+10.;
  printf("it Fstm Tcond UAtot\n");
  for(iter=0;iter<32;iter++)
    {
    mst.P=stg.swc*Fstm;
    mst.H=HofTP(mst.T,mst.P,year);
    uep.P=Psat(uep.T,year);
    Turbine(mst.P,mst.H,uep.P,stg.eff,&uep.H);
    kW.stg=Fstm*(mst.H-uep.H);
    cnd.T=Tsat(uep.P,year);
    cnd.H=Hf(cnd.T,year);
    kW.cnd=Fstm*(uep.H-cnd.H);
    cnd.T=Tcond(kW.cnd,Tccw);
    cnd.P=Psat(cnd.T,year);
    evp.P=mst.P+0.2;
    evp.T=Tsat(evp.P,year);
    evp.H=Hg(evp.T,year);
    eco.P=evp.P;
    eco.T=evp.T-5.;
    eco.H=HofTP(eco.T,eco.P,year);
    bfw.P=evp.P+0.8;
    Pump(cnd.P,cnd.H,bfw.P,bfp.eff,&bfw.H);
    kW.pmp=Fstm*(bfw.H-cnd.H);
    kW.net=gt.kWnet+kW.stg*gen.eff-kW.pmp;
    Qsh=Fstm*(mst.H-evp.H);
    Qev=Fstm*(evp.H-eco.H);
    Qec=Fstm*(eco.H-bfw.H);
    if(Qsh<=0.||Qev<=0.||Qec<=0.)
      {
      Fstm/=2.;
      continue;
      }
    T1=gt.EGT;
    T2=T1-Qsh/gt.EGW/Cp;
    T3=T2-Qev/gt.EGW/Cp;
    T4=T3-Qec/gt.EGW/Cp;
    dTsh=LMTD(T1-mst.T,T2-evp.T);
    dTev=LMTD(T2-evp.T,T3-eco.T);
    dTec=LMTD(T3-eco.T,T4-bfw.T);
    if(dTsh<=0.||dTev<=0.||dTec<=0.)
      {
      Fstm/=2.;
      continue;
      }
    UAec=Qec/dTec;
    UAev=Qev/dTev;
    UAsh=Qsh/dTsh;
    UAto=UAec+UAev+UAsh;
    printf("%2i %4.2lf %5.1lf %5.1lf\n",iter,Fstm,cnd.T,UAto);
    if(fabs(UAto/UAta-1.)<0.0001)
      if(fabs(uep.T-cnd.T)<0.5)
        break;
    Fstm=fmax(Fstm/1.5,fmin(Fstm*1.5,Fstm*(UAta/UAto+1.)/2.));
    uep.T=(uep.T+cnd.T)/2.;
    }
  printf("kWnet=%.0lf, kJ/kWh=%.0lf\n",kW.net,Qfu*3600./kW.net);
  }

int main(int argc,char**argv,char**envp)
  {
  double baro=0.101325; /* barometric pressure [MPa] */
  double Tamb=290;      /* ambient temperature [deg K] */
  double RH=0.6;        /* ambient relative humidity */
  double Tccw=290.;     /* cooling water inlet temp. [deg K] */
  CCPP(baro,Tamb,RH,Tccw);
  return(0);
  }
