/* simple three-pressure HRSG and
   Combined Cycle Power Plant Model
   using English units
   developed by Dudley J. Benton */

#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#define _USE_MATH_DEFINES
#include <math.h>
#include "..\\AllSteam\\AllSteam.h"
#include "moistair.h"

/*******************************************************************************
  GENERIC FUNCTIONS
*******************************************************************************/

static double fmin(double x,double y)
  {
  if(x<y)
    return(x);
  return(y);
  }

static double fmax(double x,double y)
  {
  if(x>y)
    return(x);
  return(y);
  }

double LMTD(double dT1,double dT2)
  {
  if(dT1<=0.||dT2<=0.)
    return(0.);
  if(fabs(dT1-dT2)<0.001)
    return(sqrt(dT1*dT2));
  return((dT1-dT2)/log(dT1/dT2));
  }

/*******************************************************************************
  EXHAUST GAS PROPERTY FUNCTIONS
*******************************************************************************/

typedef struct{double a,b,c,d,MW;}GAS;
GAS N2 ={-1.823844E-01,2.450080E-01, 9.515373E-06,1.278920E-09,28.013};
GAS O2 ={-1.173314E-02,2.177264E-01, 1.570874E-06,1.001867E-09,31.998};
GAS CO2={ 1.374508E-03,1.873375E-01, 3.953234E-06,1.303025E-10,44.010};
GAS H2O={ 9.393189E-05,4.431567E-01, 6.169783E-08,9.216964E-12,18.015};
GAS Ar ={ 7.922561E-14,1.242788E-01,-2.526427E-18,9.458965E-22,39.948};

typedef struct{double N2,O2,CO2,H2O,Ar;}MF;

double Hgas(MF mf,double T)
  {
  double HN2,HO2,HCO2,HH2O,HAr,X,Y;
  HN2 =(( N2.d*T+ N2.c)*T+ N2.b)*T+ N2.a;
  HO2 =(( O2.d*T+ O2.c)*T+ O2.b)*T+ O2.a;
  HCO2=((CO2.d*T+CO2.c)*T+CO2.b)*T+CO2.a;
  HH2O=((H2O.d*T+H2O.c)*T+H2O.b)*T+H2O.a;
  HAr =(( Ar.d*T+ Ar.c)*T+ Ar.b)*T+ Ar.a;
  X= HN2* N2.MW*mf.N2
   + HO2* O2.MW*mf.O2
   +HCO2*CO2.MW*mf.CO2
   +HH2O*H2O.MW*mf.H2O
   + HAr* Ar.MW*mf.Ar;
  Y= N2.MW*mf.N2
   + O2.MW*mf.O2
   +CO2.MW*mf.CO2
   +H2O.MW*mf.H2O
   + Ar.MW*mf.Ar;
  return(X/Y);
  }

double Tgas(MF mf,double H)
  {
  int iter;
  double T,T1,T2;
  T1=0.;
  T2=2000.;
  for(iter=0;iter<32;iter++)
    {
    T=(T1+T2)/2.;
    if(Hgas(mf,T)<H)
      T1=T;
    else
      T2=T;
    }
  return(T);
  }

MF Ambient(double baro,double Tdb,double RH)
  {
  double W,x,y;
  static MF mf;
  W=fWdbrh(baro,Tdb,RH);
  x=W/(1.+W);
  y=x*28.9645/18.01534;
  mf.H2O=y;
  mf.N2 =0.7808*(1.-y);
  mf.O2 =0.2095*(1.-y);
  mf.CO2=0.0004*(1.-y);
  mf.Ar =0.0093*(1.-y);
  return(mf);
  }

MF Combustion(MF mf1,double Fair,double Ffuel,double HCratio)
  {
  double C,H,S;
  static MF mf2;
  H=HCratio/(1.+HCratio);
  C=1.-H;
  mf2.N2 =Fair*mf1.N2;
  mf2.O2 =Fair*mf1.O2 -Ffuel*(C+H/2.);
  mf2.CO2=Fair*mf1.CO2+Ffuel*C;
  mf2.Ar =Fair*mf1.Ar;
  mf2.H2O=Fair*mf1.H2O+Ffuel*H/2.;
  S=mf2.N2+mf2.O2+mf2.CO2+mf2.Ar+mf2.H2O;
  mf2.N2 /=S;
  mf2.O2 /=S;
  mf2.CO2/=S;
  mf2.Ar /=S;
  mf2.H2O/=S;
  return(mf2);
  }

void ListMoleFractions(char*location,MF mf)
  {
  printf("%s: %6.4lf,%6.4lf,%6.4lf,%6.4lf,%6.4lf\n",location,mf.N2,mf.O2,mf.CO2,mf.Ar,mf.H2O);
  }

MF mfa; /* ambient (includes relative humidity) */
MF mf1; /* before duct burner */
MF mf2; /* after duct burner */

/*******************************************************************************
  GAS TURBINE DATA AND FUNCTIONS
*******************************************************************************/

double Bgas=19500.; /* fuel gas LHV [BTU/lbm] */
double HCratio=3.5; /* H/C ratio of fuel gas */

typedef struct{double kWnet,BTUkWh,EGW,EGT;}GT;

GT LM6000={43273., /* net power output [kW] */
            8244., /* net heat rate [BTU/kWh] */
         1000000., /* exhaust flow [lbm/hr] */
            850.}; /* exhaust temperature [deg F] */

GT GasTurbinePerformance(double P,double T,double RH)
  {
  static GT gt;
  gt.kWnet =LM6000.kWnet    *(P/14.696)*(-0.00241*(T-60.)+1.)*( 0.00655*(RH-0.6)+1.);
  gt.BTUkWh=LM6000.BTUkWh   *(14.696/P)*( 0.00071*(T-60.)+1.)*( 0.00153*(RH-0.6)+1.);
  gt.EGW   =LM6000.EGW      *(P/14.696)*(-0.00172*(T-60.)+1.)*(-0.00533*(RH-0.6)+1.);
  gt.EGT   =LM6000.EGT+0.912*(P-14.696)+  0.272  *(T-60.)    +  2.39   *(RH-0.6)    ;
  return(gt);
  }

/*******************************************************************************
  STEAM TURBINE DATA AND FUNCTIONS
*******************************************************************************/

int year=97; /* steam properties (must be 67, 69, 84, 95, or 97) */
             /* use +year for English and -year for SI units */

void Turbine(double P1,double H1,double P2,double efficiency,double*H2)
  {
  double H2S,S1;
  S1=SofPH(P1,H1,year);
  H2S=HofPS(P2,S1,year);
  *H2=H1+(H2S-H1)*efficiency;
  }

void Pump(double P1,double H1,double P2,double efficiency,double*H2)
  {
  double H2S,S1;
  S1=SofPH(P1,H1,year);
  H2S=HofPS(P2,S1,year);
  *H2=H1+(H2S-H1)/efficiency;
  }

struct{double swc,eff;}hpt={0.025263,0.82},
                       ipt={0.003509,0.82},
                       lpt={0.001003,0.82};

struct{double eff;}cbp={0.85},
                   ipp={0.85},
                   hpp={0.85};

double RH_flow_ratio=0.95; /* 5% leakage */

double GenLoss(double kWshaft)
  {
  return(100.+0.007*kWshaft);
  }

/*******************************************************************************
  CONDENSER AND HEAT EXCHANGER PERFORMANCE FUNCTIONS
*******************************************************************************/

double Tcond(double MBTUhr,double Tin)
  {
  return(Tin+MBTUhr/2E6);
  }

/* pressure drops */

struct{double lp,ip,hp,rh;}dPsh={5,10,25,10};
struct{double lp,ip,hp;}dPec={100,25,50};

/* desired temperatures (superheat and subcooling) */

struct{double lp,ip,hp1,hp2,rh1,rh2;}Tsh={500,600,700,800,700,800};
struct{double lp,ip,hp1,hp2,hp3;}dTec={5,50,150,75,20};

/* overall UA */

struct{double lp,ip,hp;}UAtarget={75E3,50E3,700E3};

/*******************************************************************************
  HRSG MODELING FUNCTIONS AND VARIABLES
*******************************************************************************/

double dThpe1,dThpe2,dThpe3,dThpev,dThps1,dThps2,dTipec,dTipev,dTipsh,dTlpec,
  dTlpev,dTlpsh,dTrh1,dTrh2,H01,H02,H03,H04,H05,H06,H07,H08,H09,H10,H11,H12,
  H13,H14,H15,H16,Hlf,Qhpe1,Qhpe2,Qhpe3,Qhpev,Qhps1,Qhps2,Qipec,Qipev,Qipsh,
  Qlpec,Qlpev,Qlpsh,Qrh1,Qrh2,T01,T02,T03,T04,T05,T06,T07,T08,T09,T10,T11,
  T12,T13,T14,T15,T16,UAhp,UAhpe1,UAhpe2,UAhpe3,UAhpev,UAhps1,UAhps2,UAip,
  UAipec,UAipev,UAipsh,UAlp,UAlpec,UAlpev,UAlpsh,UArh1,UArh2;

typedef struct{double T,P,H;}TPH;
TPH cnd,crh,hpe1,hpe2,hpe3,hpev,hpfw,hps1,hps2,ipec,ipev,ipfw,
  ipsh,ipt2,lpec,lpev,lpfw,lpsh,lpt1,lpt2,rh0,rh1,rh2;

void HRSG(GT gt,double Flp,double Fip,double Fhp,double Frh,double Qdb,double Fdb)
  {

/* heat transfer for each element */

  UAhp=UAip=UAlp=0.;
  Qlpec=(Flp+Fip+Fhp)*(lpec.H-lpfw.H);
  Qlpev=Flp*(lpev.H-lpec.H)     /* lp steam */
       +(Fip+Fhp)*(Hlf-lpec.H); /* sat lqd */
  Qlpsh=Flp*(lpsh.H-lpev.H);
  Qipec=Fip*(ipec.H-ipfw.H);
  Qipev=Fip*(ipev.H-ipec.H);
  Qipsh=Fip*(ipsh.H-ipev.H);
  Qhpe1=Fhp*(hpe1.H-hpfw.H);
  Qhpe2=Fhp*(hpe2.H-hpe1.H);
  Qhpe3=Fhp*(hpe3.H-hpe2.H);
  Qhpev=Fhp*(hpev.H-hpe3.H);
  Qhps1=Fhp*(hps1.H-hpev.H);
  Qhps2=Fhp*(hps2.H-hps1.H);
  Qrh1=(Frh+Fip)*(rh1.H-rh0.H);
  Qrh2=(Frh+Fip)*(rh2.H-rh1.H);
  if(Qlpec<=0.||Qlpev<=0.||Qlpsh<=0.||Qipec<=0.||Qipev<=0.||Qipsh<=0.||
     Qhpe1<=0.||Qhpe2<=0.||Qhpe3<=0.||Qhpev<=0.||Qhps1<=0.||Qhps2<=0.||
     Qrh1 <=0.||Qrh2 <=0.)
    return;

/* gas path temperatures and enthalpies */
/* note that mole fractions change after duct burner */

  T01=gt.EGT                ;H01=Hgas(mf1,T01); /* GT   */
  H02=H01-Qrh2 / gt.EGW     ;T02=Tgas(mf1,H02); /* RH2  */
  H03=H02-Qhps2/ gt.EGW     ;T03=Tgas(mf1,H03); /* HPS2 */
  H04=H03+Qdb  / gt.EGW     ;T04=Tgas(mf2,H04); /* DB   */
  H05=H04-Qrh1 /(gt.EGW+Fdb);T05=Tgas(mf2,H05); /* RH1  */
  H06=H05-Qhps1/(gt.EGW+Fdb);T06=Tgas(mf2,H06); /* HPS1 */
  H07=H06-Qhpev/(gt.EGW+Fdb);T07=Tgas(mf2,H07); /* HPEV */
  H08=H07-Qhpe3/(gt.EGW+Fdb);T08=Tgas(mf2,H08); /* HPE3 */
  H09=H08-Qipsh/(gt.EGW+Fdb);T09=Tgas(mf2,H09); /* IPSH */
  H10=H09-Qlpsh/(gt.EGW+Fdb);T10=Tgas(mf2,H10); /* LPSH */
  H11=H10-Qhpe2/(gt.EGW+Fdb);T11=Tgas(mf2,H11); /* HPE2 */
  H12=H11-Qipev/(gt.EGW+Fdb);T12=Tgas(mf2,H12); /* IPEV */
  H13=H12-Qhpe1/(gt.EGW+Fdb);T13=Tgas(mf2,H13); /* HPE1 */
  H14=H13-Qipec/(gt.EGW+Fdb);T14=Tgas(mf2,H14); /* IPEC */
  H15=H14-Qlpev/(gt.EGW+Fdb);T15=Tgas(mf2,H15); /* LPEV */
  H16=H15-Qlpec/(gt.EGW+Fdb);T16=Tgas(mf2,H16); /* LPEC */

/* temperature differences */

  dTrh2 =LMTD(T01- rh2.T,T02- rh1.T);
  dThps2=LMTD(T02-hps2.T,T04-hps1.T);
  dTrh1 =LMTD(T04- rh1.T,T05- rh0.T);
  dThps1=LMTD(T05-hps1.T,T06-hpev.T);
  dThpev=LMTD(T06-hpev.T,T07-hpe3.T);
  dThpe3=LMTD(T07-hpe3.T,T08-ipsh.T);
  dTipsh=LMTD(T08-ipsh.T,T09-lpsh.T);
  dTlpsh=LMTD(T09-lpsh.T,T10-hpe2.T);
  dThpe2=LMTD(T10-hpe2.T,T11-ipev.T);
  dTipev=LMTD(T11-ipev.T,T12-hpe1.T);
  dThpe1=LMTD(T12-hpe1.T,T13-ipec.T);
  dTipec=LMTD(T13-ipec.T,T14-lpev.T);
  dTlpev=LMTD(T14-lpev.T,T15-lpec.T);
  dTlpec=LMTD(T15-lpec.T,T16-lpfw.T);
  if(dTlpec<=0.||dTlpev<=0.||dTlpsh<=0.||dTipec<=0.||dTipev<=0.||dTipsh<=0.||
     dThpe1<=0.||dThpe2<=0.||dThpe3<=0.||dThpev<=0.||dThps1<=0.||dThps2<=0.||
     dTrh1 <=0.||dTrh2 <=0.)
    return;

/* conductances */

  UArh1 =Qrh1 /dTrh1 ;
  UArh2 =Qrh2 /dTrh2 ;
  UAhps1=Qhps1/dThps1;
  UAhps2=Qhps2/dThps2;
  UAhpev=Qhpev/dThpev;
  UAhpe3=Qhpe3/dThpe3;
  UAipsh=Qipsh/dTipsh;
  UAlpsh=Qlpsh/dTlpsh;
  UAhpe2=Qhpe2/dThpe2;
  UAipev=Qipev/dTipev;
  UAhpe1=Qhpe1/dThpe1;
  UAipec=Qipec/dTipec;
  UAlpev=Qlpev/dTlpev;
  UAlpec=Qlpec/dTlpec;

  UAhp=UAhpe1+UAhpe2+UAhpe3+UAhpev+UAhps1+UAhps2+UArh1+UArh2;
  UAip=UAipec+UAipev+UAipsh;
  UAlp=UAlpec+UAlpev+UAlpsh;
  }

/*******************************************************************************
  COMBINED CYCLE DATA AND FUNCTIONS
*******************************************************************************/

double d,damp,Fair,Fdb,Fgt,Fhp,Fip,Flp,Frh,HtRt,Qcnd,Qgt,Qtot,Tcnd;
struct{double cbp,hpp,hpt,gro,ipp,ipt,lpt,net,tot;}kW;

void CCPP(double baro,double Tdb,double RH,double Tccw,double Qdb,int list)
  {
  int i,iter;
  GT gt;

/* ambient air mole fractions */

  mfa=Ambient(baro,Tdb,RH);
  if(list)
    ListMoleFractions("ambient",mfa);

/* gas turbine power, heat rate, exhaust flow, and exhaust temperature */
/* (this is not an iterative calculation) */

  gt=GasTurbinePerformance(baro,Tdb,RH);
  Fgt=gt.BTUkWh*gt.kWnet/Bgas;
  Fair=gt.EGW-Fgt;
  mf1=Combustion(mfa,Fair,Fgt,HCratio);
  if(list)
    ListMoleFractions("exhaust",mf1);
  Fdb=Qdb/Bgas;
  mf2=Combustion(mf1,Fair+Fgt,Fdb,HCratio);
  if(list)
    ListMoleFractions("ductbrn",mf2);

/* initial estimates of implicit parameters */

  Flp=gt.EGW/500.;
  Fip=gt.EGW/120.;
  Fhp=gt.EGW/ 12.;
  Tcnd=Tccw+40.;
  damp=1.;
  if(list)
    printf("it   Flp   Fip    Fhp Tcn   UAlp   UAip    UAhp\n");
  for(i=iter=0;iter<128;iter++)
    {
    if(list)
      printf("%2i %5.0lf %5.0lf %6.0lf %3.0lf",iter,Flp,Fip,Fhp,Tcnd);

/* reheat flow (HP steam less leakage) */

    Frh=Fhp*RH_flow_ratio;

/* LP steam path properties */

    lpsh.P=(Frh+Flp)*lpt.swc;
    lpev.P=lpsh.P+dPsh.lp;
    lpec.P=lpev.P;
    lpfw.P=lpev.P+dPec.lp;
    lpev.T=Tsat(lpev.P,year);
    lpec.T=lpev.T-dTec.lp;
    lpsh.T=Tsh.lp;
    cnd.H=Hf(Tcnd,year);
    Pump(cnd.P,cnd.H,lpfw.P,cbp.eff,&lpfw.H);
    lpfw.T=TofPH(lpfw.P,lpfw.H,year);
    lpec.H=HofTP(lpec.T,lpec.P,year);
    lpev.H=Hg(lpev.T,year);
    lpsh.H=HofTP(lpsh.T,lpsh.P,year);
    Hlf=Hf(lpev.T,year);

/* HP steam path properties */

    hps2.P=Fhp*hpt.swc;
    hps1.P=hps2.P+dPsh.hp;
    hpev.P=hps1.P+dPsh.hp;
    hpe3.P=hpev.P;
    hpe2.P=hpe3.P+dPec.hp;
    hpe1.P=hpe2.P+dPec.hp;
    hpfw.P=hpe1.P+dPec.hp;
    hpev.T=Tsat(hpev.P,year);
    hpe3.T=hpev.T-dTec.hp3;
    hpe2.T=hpev.T-dTec.hp2;
    hpe1.T=hpev.T-dTec.hp1;
    Pump(lpev.P,Hlf,hpe1.P,hpp.eff,&hpfw.H);
    hpfw.T=TofPH(hpfw.P,hpfw.H,year);
    hps1.T=Tsh.hp1;
    hps2.T=Tsh.hp2;
    hpfw.H=HofTP(hpfw.T,hpfw.P,year);
    hpe1.H=HofTP(hpe1.T,hpe1.P,year);
    hpe2.H=HofTP(hpe2.T,hpe1.P,year);
    hpe3.H=HofTP(hpe3.T,hpe1.P,year);
    hpev.H=Hg(hpev.T,year);
    hps1.H=HofTP(hps1.T,hps1.P,year);
    hps2.H=HofTP(hps2.T,hps2.P,year);

/* RH steam path properties */

    rh2.P=Frh*ipt.swc;
    rh1.P=rh2.P+dPsh.rh;
    rh0.P=rh1.P+dPsh.rh;
    rh1.T=Tsh.rh1;
    rh1.H=HofTP(rh1.T,rh1.P,year);
    rh2.T=Tsh.rh2;
    rh2.H=HofTP(rh2.T,rh2.P,year);

/* IP steam path properties */

    ipsh.P=rh0.P;
    ipev.P=ipsh.P+dPsh.ip;
    ipec.P=ipev.P;
    ipfw.P=ipec.P+dPec.ip;
    ipev.T=Tsat(ipev.P,year);
    ipec.T=ipev.T-dTec.ip;
    Pump(lpev.P,Hlf,ipec.P,ipp.eff,&ipfw.H);
    ipfw.T=TofPH(ipec.P,ipfw.H,year);
    ipsh.T=Tsh.ip;
    ipfw.H=HofTP(ipfw.T,ipfw.P,year);
    ipec.H=HofTP(ipec.T,ipec.P,year);
    ipev.H=Hg(ipev.T,year);
    ipsh.H=HofTP(ipsh.T,ipsh.P,year);

/* mix CRH and IP steam and update T,H */

    crh.P=rh0.P;
    Turbine(hps2.P,hps2.H,crh.P,hpt.eff,&crh.H);
    rh0.H=(Frh*crh.H+Fip*ipsh.H)/(Frh+Fip);
    rh0.T=TofPH(rh0.P,rh0.H,year);

/* IPT exit/LPT inlet */

    ipt2.P=lpt1.P=lpsh.P;
    Turbine(rh2.P,rh2.H,ipt2.P,ipt.eff,&ipt2.H);
    lpt1.H=(Frh*ipt2.H+Flp*lpsh.H)/(Frh+Flp);
    lpt2.P=Psat(Tcnd,year);
    Turbine(lpt1.P,lpt1.H,lpt2.P,lpt.eff,&lpt2.H);
    Qcnd=(Frh+Flp)*(lpt2.H-cnd.H);

    HRSG(gt,Flp,Fip,Fhp,Frh,Qdb,Fdb);
    if(list)
      printf(" %6.0lf %6.0lf %7.0lf\n",UAlp,UAip,UAhp);
    if(UAlp<=0.||UAip<=0.||UAhp<=0.)
      {
      Flp/=2.;
      Fip/=2.;
      Fhp/=2.;
      if(i)
        damp+=2.;
      continue;
      }
    cnd.T=Tcond(Qcnd,Tccw);
    if(fabs(UAlp/UAtarget.lp-1.)<0.001)
      if(fabs(UAip/UAtarget.ip-1.)<0.001)
        if(fabs(UAhp/UAtarget.hp-1.)<0.001)
          if(fabs(Tcnd-cnd.T)<0.5)
            goto done;
    i++;
    d=1.+1./damp;
    Flp=fmax(Flp/d,fmin(Flp*d,Flp*(1.+UAtarget.lp/UAlp)/2.));
    Fip=fmax(Fip/d,fmin(Fip*d,Fip*(1.+UAtarget.ip/UAip)/2.));
    Fhp=fmax(Fhp/d,fmin(Fhp*d,Fhp*(1.+UAtarget.hp/UAhp)/2.));
    Tcnd=(Tcnd+cnd.T)/2.;
    }
  printf("iterations failed to converge!\n");
done:
  kW.hpt=Fhp*(hps2.H-crh.H)/3412.141633;
  kW.ipt=Frh*(ipt2.H-rh2.H)/3412.141633;
  kW.lpt=(Frh+Flp)*(lpt1.H-lpt2.H)/3412.141633;
  kW.cbp=(Frh+Flp)*(lpfw.H- cnd.H)/3412.141633;
  kW.ipp=Fip*(ipfw.H-Hlf)/3412.141633;
  kW.hpp=Fhp*(hpfw.H-Hlf)/3412.141633;
  kW.gro=kW.hpt+kW.ipt+kW.lpt;
  kW.net=kW.gro-GenLoss(kW.gro)-kW.cbp-kW.ipp-kW.hpp;
  Qgt=gt.BTUkWh*gt.kWnet;
  Qtot=Qgt+Qdb;
  kW.tot=gt.kWnet+kW.net;
  HtRt=Qtot/kW.tot;
  if(list)
    {
    printf("GT=%.0lf kW, ST=%.0lf kW, total=%.0lf kW\n",gt.kWnet,kW.net,kW.tot);
    printf("Heat Rate=%.0lf BTU/kWh\n",HtRt);
    printf("fuel: GT=%.0lf, DB=%.0lf\n",Fgt,Fdb);
    }
  }

int main(int argc,char**argv,char**envp)
  {
  double baro=14.696; /* barometric pressure [psia] */
  double Tdb=60.;     /* ambient temperature [F] */
  double RH=0.60;     /* ambient relative humidity */
  double Tccw=60.;    /* cooling water inlet temp. [F] */
  double Qdb=43E6;    /* duct burner heat input [BTU/hr] */
  printf("three-pressure combined cycle power plant model\n");
  CCPP(baro,Tdb,RH,Tccw,Qdb,1);
  for(baro=13.696;baro<15.697;baro+=0.1)
    {
    CCPP(baro,60.,0.6,Tccw,Qdb,0);
    printf("%.3lf,%.0lf,%.0lf\n",baro,kW.tot,HtRt);
    }
  for(Tdb=0.;Tdb<121.;Tdb+=5.)
    {
    CCPP(14.696,Tdb,0.6,Tccw,Qdb,0);
    printf("%.0lf,%.0lf,%.0lf\n",Tdb,kW.tot,HtRt);
    }
  for(RH=0.;RH<1.01;RH+=0.05)
    {
    CCPP(14.696,60.,RH,Tccw,Qdb,0);
    printf("%.2lf,%.0lf,%.0lf\n",RH,kW.tot,HtRt);
    }
  return(0);
  }
