/* simple three-pressure HRSG
   using English units
   developed by Dudley J. Benton */

#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#define _USE_MATH_DEFINES
#include <math.h>
#include "..\\AllSteam\\AllSteam.h"

/*******************************************************************************
  USER DATA STATEMENTS
*******************************************************************************/

int year=97; /* steam properties (must be 67, 69, 84, 95, or 97) */
             /* use +year for English and -year for SI units */

/* GT exhaust mole fractions */

struct{double N2,O2,CO2,H2O,Ar,SO2;}mf={0.746,0.124,0.038,0.083,0.009,0.000};

/* GT exhaust temperature and flow */

struct{double F,T;}GT={1000000,1050};

/* feedwater inlet temperature */

double Tfw=60.;

/* steam outlet pressure (controlled by steam turbine swallowing capacity) */

struct{double hp,ip,lp;}Pstm={1800,250,75};

/* pressure drops */

struct{double lp,ip,hp;}dPsh={5,10,25};
struct{double lp,ip,hp1,hp2,hp3;}dPec={100,25,50,50,50};

/* desired temperature differences (superheat and subcooling) */

struct{double lp,ip,hp;}dTsh={175,200,275};
struct{double lp,ip,hp1,hp2,hp3;}dTec={5,50,150,75,20};

/* overall UA */

struct{double lp,ip,hp;}UAtarget={20E4,5E4,50E4};

/*******************************************************************************
  GENERIC FUNCTIONS
*******************************************************************************/

static double fmin(double x,double y)
  {
  if(x<y)
    return(x);
  return(y);
  }

static double fmax(double x,double y)
  {
  if(x>y)
    return(x);
  return(y);
  }

double LMTD(double dT1,double dT2)
  {
  if(dT1<=0.||dT2<=0.)
    return(0.);
  if(fabs(dT1-dT2)<0.001)
    return(sqrt(dT1*dT2));
  return((dT1-dT2)/log(dT1/dT2));
  }

/*******************************************************************************
  EXHAUST GAS PROPERTY FUNCTIONS
*******************************************************************************/

typedef struct{double a,b,c,d,MW;}GAS;
GAS N2 ={-1.823844E-01,2.450080E-01, 9.515373E-06,1.278920E-09,28.013};
GAS O2 ={-1.173314E-02,2.177264E-01, 1.570874E-06,1.001867E-09,31.998};
GAS CO2={ 1.374508E-03,1.873375E-01, 3.953234E-06,1.303025E-10,44.010};
GAS H2O={ 9.393189E-05,4.431567E-01, 6.169783E-08,9.216964E-12,18.015};
GAS Ar ={ 7.922561E-14,1.242788E-01,-2.526427E-18,9.458965E-22,39.948};
GAS SO2={ 1.495539E-05,1.417236E-01, 1.931810E-08,1.479716E-12,64.066};

double Hgas(double T)
  {
  double HN2,HO2,HCO2,HH2O,HAr,HSO2,X,Y;
  HN2 =(( N2.d*T+ N2.c)*T+ N2.b)*T+ N2.a;
  HO2 =(( O2.d*T+ O2.c)*T+ O2.b)*T+ O2.a;
  HCO2=((CO2.d*T+CO2.c)*T+CO2.b)*T+CO2.a;
  HH2O=((H2O.d*T+H2O.c)*T+H2O.b)*T+H2O.a;
  HAr =(( Ar.d*T+ Ar.c)*T+ Ar.b)*T+ Ar.a;
  HSO2=((SO2.d*T+SO2.c)*T+SO2.b)*T+SO2.a;
  X= HN2* N2.MW*mf.N2
   + HO2* O2.MW*mf.O2
   +HCO2*CO2.MW*mf.CO2
   +HH2O*H2O.MW*mf.H2O
   + HAr* Ar.MW*mf.Ar
   +HSO2*SO2.MW*mf.SO2;
  Y= N2.MW*mf.N2
   + O2.MW*mf.O2
   +CO2.MW*mf.CO2
   +H2O.MW*mf.H2O
   + Ar.MW*mf.Ar
   +SO2.MW*mf.SO2;
  return(X/Y);
  }

double Tgas(double H)
  {
  int iter;
  double T,T1,T2;
  T1=0.;
  T2=2000.;
  for(iter=0;iter<32;iter++)
    {
    T=(T1+T2)/2.;
    if(Hgas(T)<H)
      T1=T;
    else
      T2=T;
    }
  return(T);
  }


/*******************************************************************************
  HRSG MODELING FUNCTIONS AND VARIABLES
*******************************************************************************/

double dThpe1,dThpe2,dThpe3,dThpev,dThpsh,dTipec,dTipev,dTipsh,dTlpec,dTlpev,
  dTlpsh,H01,H02,H03,H04,H05,H06,H07,H08,H09,H10,H11,H12,Hlf,Qhpe1,Qhpe2,Qhpe3,
  Qhpev,Qhpsh,Qipec,Qipev,Qipsh,Qlpec,Qlpev,Qlpsh,T01,T02,T03,T04,T05,T06,T07,
  T08,T09,T10,T11,T12,UAhp,UAhpe1,UAhpe2,UAhpe3,UAhpev,UAhpsh,UAip,UAipec,
  UAipev,UAipsh,UAlp,UAlpec,UAlpev,UAlpsh;

struct{double H,P,T;}hpe1,hpe2,hpe3,hpev,hpfw,hpsh,ipec,ipev,ipfw,
  ipsh,lpec,lpev,lpfw,lpsh;

void Loop(double Flp,double Fip,double Fhp)
  {
  UAhp=UAip=UAlp=0.;

/* heat transfer for each element */

  Qlpec=(Flp+Fip+Fhp)*(lpec.H-lpfw.H);
  Qlpev=Flp*(lpev.H-lpec.H)     /* lp steam */
       +(Fip+Fhp)*(Hlf-lpec.H); /* sat lqd */
  Qlpsh=Flp*(lpsh.H-lpev.H);
  Qipec=Fip*(ipec.H-ipfw.H);
  Qipev=Fip*(ipev.H-ipec.H);
  Qipsh=Fip*(ipsh.H-ipev.H);
  Qhpe1=Fhp*(hpe1.H-hpfw.H);
  Qhpe2=Fhp*(hpe2.H-hpe1.H);
  Qhpe3=Fhp*(hpe3.H-hpe2.H);
  Qhpev=Fhp*(hpev.H-hpe3.H);
  Qhpsh=Fhp*(hpsh.H-hpev.H);
  if(Qlpec<=0.||Qlpev<=0.||Qlpsh<=0.||Qipec<=0.||Qipev<=0.||Qipsh<=0.||
     Qhpe1<=0.||Qhpe2<=0.||Qhpe3<=0.||Qhpev<=0.||Qhpsh<=0.)
    return;

/* gas path enthalpies */

  H01=Hgas(GT.T);
  H02=H01-Qhpsh/GT.F;
  H03=H02-Qhpev/GT.F;
  H04=H03-Qhpe3/GT.F;
  H05=H04-Qipsh/GT.F;
  H06=H05-Qlpsh/GT.F;
  H07=H06-Qhpe2/GT.F;
  H08=H07-Qipev/GT.F;
  H09=H08-Qhpe1/GT.F;
  H10=H09-Qipec/GT.F;
  H11=H10-Qlpev/GT.F;
  H12=H11-Qipec/GT.F;

/* gas path temperatures */

  T01=GT.T;
  T02=Tgas(H02);
  T03=Tgas(H03);
  T04=Tgas(H04);
  T05=Tgas(H05);
  T06=Tgas(H06);
  T07=Tgas(H07);
  T08=Tgas(H08);
  T09=Tgas(H09);
  T10=Tgas(H10);
  T11=Tgas(H11);
  T12=Tgas(H12);

/* temperature differences */

  dThpsh=LMTD(T01-hpsh.T,T02-hpev.T);
  dThpev=LMTD(T02-hpev.T,T03-hpe3.T);
  dThpe3=LMTD(T03-hpe3.T,T04-ipsh.T);
  dTipsh=LMTD(T04-ipsh.T,T05-lpsh.T);
  dTlpsh=LMTD(T05-lpsh.T,T06-hpe2.T);
  dThpe2=LMTD(T06-hpe2.T,T07-ipev.T);
  dTipev=LMTD(T07-ipev.T,T08-hpe1.T);
  dThpe1=LMTD(T08-hpe1.T,T09-ipec.T);
  dTipec=LMTD(T09-ipec.T,T10-lpev.T);
  dTlpev=LMTD(T10-lpev.T,T11-lpec.T);
  dTlpec=LMTD(T11-lpec.T,T12-lpfw.T);
  if(dTlpec<=0.||dTlpev<=0.||dTlpsh<=0.||dTipec<=0.||dTipev<=0.||dTipsh<=0.||
     dThpe1<=0.||dThpe2<=0.||dThpe3<=0.||dThpev<=0.||dThpsh<=0.)
    return;

/* conductances */

  UAhpsh=Qhpsh/dThpsh;
  UAhpev=Qhpev/dThpev;
  UAhpe3=Qhpe3/dThpe3;
  UAipsh=Qipsh/dTipsh;
  UAlpsh=Qlpsh/dTlpsh;
  UAhpe2=Qhpe2/dThpe2;
  UAipev=Qipev/dTipev;
  UAhpe1=Qhpe1/dThpe1;
  UAipec=Qipec/dTipec;
  UAlpev=Qlpev/dTlpev;
  UAlpec=Qlpec/dTlpec;

  UAhp=UAhpe1+UAhpe2+UAhpe3+UAhpev+UAhpsh;
  UAip=UAipec+UAipev+UAipsh;
  UAlp=UAlpec+UAlpev+UAlpsh;
  }

int randbetween(int lo,int hi)
  {
  return(lo+rand()%(hi-lo+1));
  }

int Solver(int randomize)
  {
  int i,iter;
  double d,damp,Fhp,Fip,Flp;

/* LP steam path properties */

  lpsh.P=Pstm.lp;
  lpev.P=lpsh.P+dPsh.lp;
  lpec.P=lpev.P;
  lpfw.P=lpev.P+dPec.lp;
  lpev.T=Tsat(lpev.P,year);
  lpec.T=lpev.T-dTec.lp;
  lpfw.T=Tfw;
  lpsh.T=lpec.T+dTsh.lp;
  lpfw.H=HofTP(lpfw.T,lpfw.P,year);
  lpec.H=HofTP(lpec.T,lpec.P,year);
  lpev.H=Hg(lpev.T,year);
  lpsh.H=HofTP(lpsh.T,lpsh.P,year);
  Hlf=Hf(lpev.T,year);

/* IP steam path properties */

  ipsh.P=Pstm.ip;
  ipev.P=ipsh.P+dPsh.ip;
  ipec.P=ipev.P;
  ipfw.P=ipec.P+dPec.ip;
  ipev.T=Tsat(ipev.P,year);
  ipec.T=ipev.T-dTec.ip;
  ipfw.T=lpev.T+1.; /* pump inefficiency */
  ipsh.T=ipev.T+dTsh.ip;
  ipfw.H=HofTP(ipfw.T,ipfw.P,year);
  ipec.H=HofTP(ipec.T,ipec.P,year);
  ipev.H=Hg(ipev.T,year);
  ipsh.H=HofTP(ipsh.T,ipsh.P,year);

/* HP steam path properties */

  hpsh.P=Pstm.hp;
  hpev.P=hpsh.P+dPsh.hp;
  hpe3.P=hpev.P;
  hpe2.P=hpe3.P+dPec.hp3;
  hpe1.P=hpe2.P+dPec.hp2;
  hpfw.P=hpe1.P+dPec.hp1;
  hpev.T=Tsat(hpev.P,year);
  hpe3.T=hpev.T-dTec.hp3;
  hpe2.T=hpev.T-dTec.hp2;
  hpe1.T=hpev.T-dTec.hp1;
  hpfw.T=lpev.T+5.; /* pump inefficiency */
  hpsh.T=hpev.T+dTsh.hp;
  hpfw.H=HofTP(hpfw.T,hpfw.P,year);
  hpe1.H=HofTP(hpe1.T,hpe1.P,year);
  hpe2.H=HofTP(hpe2.T,hpe1.P,year);
  hpe3.H=HofTP(hpe3.T,hpe1.P,year);
  hpev.H=Hg(hpev.T,year);
  hpsh.H=HofTP(hpsh.T,hpsh.P,year);

  if(randomize)
    {
    Flp=100*randbetween(1,10000);
    Fip=100*randbetween(1,10000);
    Fhp=100*randbetween(1,10000);
    }
  else
    {
    Flp=GT.F/200.;
    Fip=GT.F/200.;
    Fhp=GT.F/ 20.;
    }
  damp=1.;
  printf("it  Flp   Fip    Fhp   UAlp   UAip   UAhp\n");
  for(i=iter=0;iter<128;iter++)
    {
    d=1.+1./damp;
    printf("%2i %5.0lf %5.0lf %6.0lf",iter,Flp,Fip,Fhp);
    Loop(Flp,Fip,Fhp);
    printf(" %6.0lf %6.0lf %6.0lf\n",UAlp,UAip,UAhp);
    if(UAlp<=0.||UAip<=0.||UAhp<=0.)
      {
      Flp/=2.;
      Fip/=2.;
      Fhp/=2.;
      if(i)
        damp+=2.;
      continue;
      }
    if(fabs(UAlp/UAtarget.lp-1.)<0.0001)
      if(fabs(UAip/UAtarget.ip-1.)<0.0001)
        if(fabs(UAhp/UAtarget.hp-1.)<0.0001)
          return(1);
    i++;
    d=1.+1./damp;
    Flp=fmax(Flp/d,fmin(Flp*d,Flp*(1.+UAtarget.lp/UAlp)/2.));
    Fip=fmax(Fip/d,fmin(Fip*d,Fip*(1.+UAtarget.ip/UAip)/2.));
    Fhp=fmax(Fhp/d,fmin(Fhp*d,Fhp*(1.+UAtarget.hp/UAhp)/2.));
    }
  printf("iterations failed to converge!\n");
  return(0);
  }

int main(int argc,char**argv,char**envp)
  {
  do{;}while(Solver(1));
  return(0);
  }
