/* simple three-pressure HRSG
   using English units
   developed by Dudley J. Benton */

#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#define _USE_MATH_DEFINES
#include <math.h>
#include "..\\AllSteam\\AllSteam.h"

/*******************************************************************************
  USER DATA STATEMENTS
*******************************************************************************/

int year=97; /* steam properties (must be 67, 69, 84, 95, or 97) */
             /* use +year for English and -year for SI units */

/* GT exhaust and duct burner outlet mole fractions */

typedef struct{double N2,O2,CO2,H2O,Ar,SO2;}MF;
MF gt={0.746,0.124,0.038,0.083,0.009,0.000};
MF db={0.744,0.119,0.040,0.088,0.009,0.000};

/* GT exhaust temperature and flow */

struct{double F,T;}GT={3000000,1100};
double dTdb=100.;

/* feedwater inlet temperature */

double Tfw=60.;

/* steam outlet pressure (controlled by steam turbine swallowing capacity) */

struct{double hp,rh,lp;}Pstm={2400,600,100};
double HPT_enthalpy_drop=200;
double RH_flow_ratio=0.95;

/* pressure drops */

struct{double lp,ip,hp,rh;}dPsh={5,10,25,10};
struct{double lp,ip,hp;}dPec={100,25,50};

/* desired temperatures (superheat and subcooling) */

struct{double lp,ip,hp1,hp2,rh1,rh2;}Tsh={500,600,900,975,900,975};
struct{double lp,ip,hp1,hp2,hp3;}dTec={5,50,150,75,20};

/* overall UA */

struct{double lp,ip,hp;}UAtarget={50E4,5E4,150E4};

/*******************************************************************************
  GENERIC FUNCTIONS
*******************************************************************************/

static double fmin(double x,double y)
  {
  if(x<y)
    return(x);
  return(y);
  }

static double fmax(double x,double y)
  {
  if(x>y)
    return(x);
  return(y);
  }

double LMTD(double dT1,double dT2)
  {
  if(dT1<=0.||dT2<=0.)
    return(0.);
  if(fabs(dT1-dT2)<0.001)
    return(sqrt(dT1*dT2));
  return((dT1-dT2)/log(dT1/dT2));
  }

/*******************************************************************************
  EXHAUST GAS PROPERTY FUNCTIONS
*******************************************************************************/

typedef struct{double a,b,c,d,MW;}GAS;
GAS N2 ={-1.823844E-01,2.450080E-01, 9.515373E-06,1.278920E-09,28.013};
GAS O2 ={-1.173314E-02,2.177264E-01, 1.570874E-06,1.001867E-09,31.998};
GAS CO2={ 1.374508E-03,1.873375E-01, 3.953234E-06,1.303025E-10,44.010};
GAS H2O={ 9.393189E-05,4.431567E-01, 6.169783E-08,9.216964E-12,18.015};
GAS Ar ={ 7.922561E-14,1.242788E-01,-2.526427E-18,9.458965E-22,39.948};
GAS SO2={ 1.495539E-05,1.417236E-01, 1.931810E-08,1.479716E-12,64.066};

double Hgas(MF mf,double T)
  {
  double HN2,HO2,HCO2,HH2O,HAr,HSO2,X,Y;
  HN2 =(( N2.d*T+ N2.c)*T+ N2.b)*T+ N2.a;
  HO2 =(( O2.d*T+ O2.c)*T+ O2.b)*T+ O2.a;
  HCO2=((CO2.d*T+CO2.c)*T+CO2.b)*T+CO2.a;
  HH2O=((H2O.d*T+H2O.c)*T+H2O.b)*T+H2O.a;
  HAr =(( Ar.d*T+ Ar.c)*T+ Ar.b)*T+ Ar.a;
  HSO2=((SO2.d*T+SO2.c)*T+SO2.b)*T+SO2.a;
  X= HN2* N2.MW*mf.N2
   + HO2* O2.MW*mf.O2
   +HCO2*CO2.MW*mf.CO2
   +HH2O*H2O.MW*mf.H2O
   + HAr* Ar.MW*mf.Ar
   +HSO2*SO2.MW*mf.SO2;
  Y= N2.MW*mf.N2
   + O2.MW*mf.O2
   +CO2.MW*mf.CO2
   +H2O.MW*mf.H2O
   + Ar.MW*mf.Ar
   +SO2.MW*mf.SO2;
  return(X/Y);
  }

double Tgas(MF mf,double H)
  {
  int iter;
  double T,T1,T2;
  T1=0.;
  T2=2000.;
  for(iter=0;iter<32;iter++)
    {
    T=(T1+T2)/2.;
    if(Hgas(mf,T)<H)
      T1=T;
    else
      T2=T;
    }
  return(T);
  }


/*******************************************************************************
  HRSG MODELING FUNCTIONS AND VARIABLES
*******************************************************************************/

double dThpe1,dThpe2,dThpe3,dThpev,dThps1,dThps2,dTipec,dTipev,dTipsh,dTlpec,
  dTlpev,dTlpsh,dTrh1,dTrh2,H01,H02,H03,H04,H05,H06,H07,H08,H09,H10,H11,H12,
  H13,H14,H15,H16,Hlf,Qhpe1,Qhpe2,Qhpe3,Qhpev,Qhps1,Qhps2,Qipec,Qipev,Qipsh,
  Qlpec,Qlpev,Qlpsh,Qrh1,Qrh2,T01,T02,T03,T04,T05,T06,T07,T08,T09,T10,T11,
  T12,T13,T14,T15,T16,UAhp,UAhpe1,UAhpe2,UAhpe3,UAhpev,UAhps1,UAhps2,UAip,
  UAipec,UAipev,UAipsh,UAlp,UAlpec,UAlpev,UAlpsh,UArh1,UArh2;

struct{double H,P,T;}hpe1,hpe2,hpe3,hpev,hpfw,hps1,hps2,ipec,ipev,ipfw,
  ipsh,lpec,lpev,lpfw,lpsh,rh0,rh1,rh2;

void Loop(double Flp,double Fip,double Fhp,double Frh)
  {

/* heat transfer for each element */

  UAhp=UAip=UAlp=0.;
  Qlpec=(Flp+Fip+Fhp)*(lpec.H-lpfw.H);
  Qlpev=Flp*(lpev.H-lpec.H)     /* lp steam */
       +(Fip+Fhp)*(Hlf-lpec.H); /* sat lqd */
  Qlpsh=Flp*(lpsh.H-lpev.H);
  Qipec=Fip*(ipec.H-ipfw.H);
  Qipev=Fip*(ipev.H-ipec.H);
  Qipsh=Fip*(ipsh.H-ipev.H);
  Qhpe1=Fhp*(hpe1.H-hpfw.H);
  Qhpe2=Fhp*(hpe2.H-hpe1.H);
  Qhpe3=Fhp*(hpe3.H-hpe2.H);
  Qhpev=Fhp*(hpev.H-hpe3.H);
  Qhps1=Fhp*(hps1.H-hpev.H);
  Qhps2=Fhp*(hps2.H-hps1.H);
  Qrh1=(Frh+Fip)*(rh1.H-rh0.H);
  Qrh2=(Frh+Fip)*(rh2.H-rh1.H);
  if(Qlpec<=0.||Qlpev<=0.||Qlpsh<=0.||Qipec<=0.||Qipev<=0.||Qipsh<=0.||
     Qhpe1<=0.||Qhpe2<=0.||Qhpe3<=0.||Qhpev<=0.||Qhps1<=0.||Qhps2<=0.||
     Qrh1 <=0.||Qrh2 <=0.)
    return;

/* gas path temperatures and enthalpies */
/* note that mole fractions change after duct burner */

  T01=GT.T          ;H01=Hgas(gt,T01); /* GT   */
  H02=H01-Qrh2 /GT.F;T02=Tgas(gt,H02); /* RH2  */
  H03=H02-Qhps2/GT.F;T03=Tgas(gt,H03); /* HPS2 */
  T04=T03+dTdb      ;H04=Hgas(db,T04); /* DB   */
  H05=H04-Qrh1 /GT.F;T05=Tgas(db,H05); /* RH1  */
  H06=H05-Qhps1/GT.F;T06=Tgas(db,H06); /* HPS1 */
  H07=H06-Qhpev/GT.F;T07=Tgas(db,H07); /* HPEV */
  H08=H07-Qhpe3/GT.F;T08=Tgas(db,H08); /* HPE3 */
  H09=H08-Qipsh/GT.F;T09=Tgas(db,H09); /* IPSH */
  H10=H09-Qlpsh/GT.F;T10=Tgas(db,H10); /* LPSH */
  H11=H10-Qhpe2/GT.F;T11=Tgas(db,H11); /* HPE2 */
  H12=H11-Qipev/GT.F;T12=Tgas(db,H12); /* IPEV */
  H13=H12-Qhpe1/GT.F;T13=Tgas(db,H13); /* HPE1 */
  H14=H13-Qipec/GT.F;T14=Tgas(db,H14); /* IPEC */
  H15=H14-Qlpev/GT.F;T15=Tgas(db,H15); /* LPEV */
  H16=H15-Qlpec/GT.F;T16=Tgas(db,H16); /* LPEC */

/* temperature differences */

  dTrh2 =LMTD(T01- rh2.T,T02- rh1.T);
  dThps2=LMTD(T02-hps2.T,T04-hps1.T);
  dTrh1 =LMTD(T04- rh1.T,T05- rh0.T);
  dThps1=LMTD(T05-hps1.T,T06-hpev.T);
  dThpev=LMTD(T06-hpev.T,T07-hpe3.T);
  dThpe3=LMTD(T07-hpe3.T,T08-ipsh.T);
  dTipsh=LMTD(T08-ipsh.T,T09-lpsh.T);
  dTlpsh=LMTD(T09-lpsh.T,T10-hpe2.T);
  dThpe2=LMTD(T10-hpe2.T,T11-ipev.T);
  dTipev=LMTD(T11-ipev.T,T12-hpe1.T);
  dThpe1=LMTD(T12-hpe1.T,T13-ipec.T);
  dTipec=LMTD(T13-ipec.T,T14-lpev.T);
  dTlpev=LMTD(T14-lpev.T,T15-lpec.T);
  dTlpec=LMTD(T15-lpec.T,T16-lpfw.T);
  if(dTlpec<=0.||dTlpev<=0.||dTlpsh<=0.||dTipec<=0.||dTipev<=0.||dTipsh<=0.||
     dThpe1<=0.||dThpe2<=0.||dThpe3<=0.||dThpev<=0.||dThps1<=0.||dThps2<=0.||
     dTrh1 <=0.||dTrh2 <=0.)
    return;

/* conductances */

  UArh1 =Qrh1 /dTrh1 ;
  UArh2 =Qrh2 /dTrh2 ;
  UAhps1=Qhps1/dThps1;
  UAhps2=Qhps2/dThps2;
  UAhpev=Qhpev/dThpev;
  UAhpe3=Qhpe3/dThpe3;
  UAipsh=Qipsh/dTipsh;
  UAlpsh=Qlpsh/dTlpsh;
  UAhpe2=Qhpe2/dThpe2;
  UAipev=Qipev/dTipev;
  UAhpe1=Qhpe1/dThpe1;
  UAipec=Qipec/dTipec;
  UAlpev=Qlpev/dTlpev;
  UAlpec=Qlpec/dTlpec;

  UAhp=UAhpe1+UAhpe2+UAhpe3+UAhpev+UAhps1+UAhps2+UArh1+UArh2;
  UAip=UAipec+UAipev+UAipsh;
  UAlp=UAlpec+UAlpev+UAlpsh;
  }

int randbetween(int lo,int hi)
  {
  return(lo+rand()%(hi-lo+1));
  }

int Solver(int randomize)
  {
  int i,iter;
  double d,damp,Fhp,Fip,Flp,Frh;

/* LP steam path properties */

  lpsh.P=Pstm.lp;
  lpev.P=lpsh.P+dPsh.lp;
  lpec.P=lpev.P;
  lpfw.P=lpev.P+dPec.lp;
  lpev.T=Tsat(lpev.P,year);
  lpec.T=lpev.T-dTec.lp;
  lpfw.T=Tfw;
  lpsh.T=Tsh.lp;
  lpfw.H=HofTP(lpfw.T,lpfw.P,year);
  lpec.H=HofTP(lpec.T,lpec.P,year);
  lpev.H=Hg(lpev.T,year);
  lpsh.H=HofTP(lpsh.T,lpsh.P,year);
  Hlf=Hf(lpev.T,year);

/* HP steam path properties */

  hps2.P=Pstm.hp;
  hps1.P=hps2.P+dPsh.hp;
  hpev.P=hps1.P+dPsh.hp;
  hpe3.P=hpev.P;
  hpe2.P=hpe3.P+dPec.hp;
  hpe1.P=hpe2.P+dPec.hp;
  hpfw.P=hpe1.P+dPec.hp;
  hpev.T=Tsat(hpev.P,year);
  hpe3.T=hpev.T-dTec.hp3;
  hpe2.T=hpev.T-dTec.hp2;
  hpe1.T=hpev.T-dTec.hp1;
  hpfw.T=lpev.T+5.; /* pump inefficiency */
  hps1.T=Tsh.hp1;
  hps2.T=Tsh.hp2;
  hpfw.H=HofTP(hpfw.T,hpfw.P,year);
  hpe1.H=HofTP(hpe1.T,hpe1.P,year);
  hpe2.H=HofTP(hpe2.T,hpe1.P,year);
  hpe3.H=HofTP(hpe3.T,hpe1.P,year);
  hpev.H=Hg(hpev.T,year);
  hps1.H=HofTP(hps1.T,hps1.P,year);
  hps2.H=HofTP(hps2.T,hps2.P,year);

/* RH steam path properties */

  rh2.P=Pstm.rh;
  rh1.P=rh2.P+dPsh.rh;
  rh0.P=rh1.P+dPsh.rh;
  rh1.T=Tsh.rh1;
  rh1.H=HofTP(rh1.T,rh1.P,year);
  rh2.T=Tsh.rh2;
  rh2.H=HofTP(rh2.T,rh2.P,year);

/* IP steam path properties */

  ipsh.P=rh0.P;
  ipev.P=ipsh.P+dPsh.ip;
  ipec.P=ipev.P;
  ipfw.P=ipec.P+dPec.ip;
  ipev.T=Tsat(ipev.P,year);
  ipec.T=ipev.T-dTec.ip;
  ipfw.T=lpev.T+1.; /* pump inefficiency */
  ipsh.T=Tsh.ip;
  ipfw.H=HofTP(ipfw.T,ipfw.P,year);
  ipec.H=HofTP(ipec.T,ipec.P,year);
  ipev.H=Hg(ipev.T,year);
  ipsh.H=HofTP(ipsh.T,ipsh.P,year);

  if(randomize)
    {
    Flp=100*randbetween(1,10000);
    Fip=100*randbetween(1,10000);
    Fhp=100*randbetween(1,10000);
    }
  else
    {
    Flp=GT.F/200.;
    Fip=GT.F/200.;
    Fhp=GT.F/ 20.;
    }
  damp=1.;
  printf("it  Flp   Fip    Fhp   UAlp   UAip    UAhp\n");
  for(i=iter=0;iter<128;iter++)
    {
    d=1.+1./damp;
    printf("%2i %5.0lf %5.0lf %6.0lf",iter,Flp,Fip,Fhp);

/* reheat flow (HP steam less leakage) */
/* mix CRH and IP steam and update T,H */

    Frh=Fhp*RH_flow_ratio;
    rh0.H=(Frh*(hps2.H-HPT_enthalpy_drop)+Fip*ipsh.H)/(Frh+Fip);
    rh0.T=TofPH(rh0.P,rh0.H,year);

    Loop(Flp,Fip,Fhp,Frh);
    printf(" %6.0lf %6.0lf %7.0lf\n",UAlp,UAip,UAhp);
    if(UAlp<=0.||UAip<=0.||UAhp<=0.)
      {
      Flp/=2.;
      Fip/=2.;
      Fhp/=2.;
      if(i)
        damp+=2.;
      continue;
      }
    if(fabs(UAlp/UAtarget.lp-1.)<0.0001)
      if(fabs(UAip/UAtarget.ip-1.)<0.0001)
        if(fabs(UAhp/UAtarget.hp-1.)<0.0001)
          return(1);
    i++;
    d=1.+1./damp;
    Flp=fmax(Flp/d,fmin(Flp*d,Flp*(1.+UAtarget.lp/UAlp)/2.));
    Fip=fmax(Fip/d,fmin(Fip*d,Fip*(1.+UAtarget.ip/UAip)/2.));
    Fhp=fmax(Fhp/d,fmin(Fhp*d,Fhp*(1.+UAtarget.hp/UAhp)/2.));
    }
  printf("iterations failed to converge!\n");
  return(0);
  }

int main(int argc,char**argv,char**envp)
  {
//do{;}while(Solver(1));
  Solver(0);
  return(0);
  }
