/* simple Rankine cycle using English units
   developed by Dudley J. Benton
*/

#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <ctype.h>
#define _USE_MATH_DEFINES
#include <float.h>
#include <math.h>
#include "..\\..\\AllSteam\\AllSteam.h"

int year=97; /* steam properties (must be 67, 69, 84, 95, or 97) */

double Aarea=108.; /* annulus area in [ft^2] */

/* user-defined turbine flow factors (pressure/flow) */

double ff1=0.00171000;
double ff2=0.00028500;
double ff3=0.00004250;
double ff4=0.00000675;

/* user-defined (fractional) pressure drops */

double dPbo=0.15;
double dP12=0.05;
double dP34=0.05;
double dP56=0.05;
double dP78=0.05;

/* user-defined enthalpy drops [BTU/lbm] */

double dH12=1.0;
double dH34=0.5;
double dH56=0.5;
double dH78=0.5;

/* user-defined heater variables [deg F] */

double TTD1=10.;
double TTD3=10.;
double DCA1= 5.;
double DCA3= 5.;

/* user-defined performance functions */

double GenLoss(double kWnet,double PF)
  {
  double x;
  x=kWnet/pow(150000.,1.5);
  return((1500.*x-65.)*x+850.);
  }

double BlrEff(double pph)
  {
  double x;
  x=pph/2.204622622/3600./115.;
  return((((-0.794407*x+1.84942)*x-1.68)*x+1.04333)*x+0.453681);
  }

double STeff(double flow) /* could have different efficiency for each section */
  {
  double x;
  x=flow/2.204622622/3600.;
  return(((-8.288101141E-7*x+0.0001991260511)*x-0.01082475795)*x+0.7701450149);
  }

double Exloss(double fps) /* exhaust loss in [BTU/lbm] */
  {
  return((((0.095908075*fps-108.1160204)*fps+38062.85256)*fps-1784140.38)/fps/fps);
  }

/* generic cycle functions (not specific to any particular system) */

void Turbine(double P1,double H1,double P2,double efficiency,double*H2)
  {
  double H2S,S1;
  S1=SofPH(P1,H1,year);
  H2S=HofPS(P2,S1,year);
  *H2=H1+(H2S-H1)*efficiency;
  }

void Pump(double P1,double H1,double P2,double efficiency,double*H2)
  {
  double H2S,S1;
  S1=SofPH(P1,H1,year);
  H2S=HofPS(P2,S1,year);
  *H2=H1+(H2S-H1)/efficiency;
  }

double density(double Ps,double h)
  {
  double hf,hg,Ts,v,vf,vg,x;
  Ts=Tsat(Ps,year);
  hf=Hf(Ts,year);
  hg=Hg(Ts,year);
  x=(h-hf)/(hg-hf);
  vf=Vf(Ts,year);
  vg=Vg(Ts,year);
  v=vf+x*(vg-vf);
  return(1./v);
  }

double AnnulusVelocity(double flow,double P,double H) /* [ft/sec] */
  {
  return(flow/Aarea/density(P,H)/3600.);
  }

void FWH(double F5,double P5,double H5,double P1,double H1,double TTD,double DCA,double*F1,double*H4,double*H8)
  {
  double T4,T5,T8,Ts;
  T5=TofPH(P5,H5,year);
  Ts=Tsat(P1,year);
  T4=T5+DCA;
  *H4=HofTP(T4,P1,year);
  T8=Ts-TTD;
  *H8=HofTP(T8,P5,year);
  *F1=F5*(*H8-H5)/(H1-*H4);
  }

double Pcond(double duty,double Tin)
  {
  double Ts;
  Ts=Tin+32.*duty/6E8;
  return(Psat(Ts,year));
  }

void Deaerator(double Ffw,double Hfw,double Pex,double Hex,double*Fex,double*Hdr)
  {
  double Ts;
  Ts=Tsat(Pex,year);
  *Hdr=Hf(Ts,year);
  *Fex=Ffw*(*Hdr-Hfw)/(Hex-*Hdr);
  }

void SolveCycle(double F1,double T1,double F20,double T20,double PF)
  {
  int converged,iter;
  double F3,F4,F5,F6,F7,F8,F9,F11;
  double H1,H2,H3,H4,H5,H6,H7,H8,H9,H10,H11,H12,H13,H14,H15,H16,H17,H18;
  double P1,P2,P3,P4,P5,P6,P7,P8,P9,P10,P12,P16;
  double T2,T4,T6,T8;
  double c2,c3,c5,c7,kWgross,kWnet,Qgross,Qnet,Van;

  printf("F1=%.0lf, T1=%.1lf, F20=%.0lf, T20=%.0lf, PF=%.3lf\n",F1,T1,F20,T20,PF);

/* initialize implicit variables */

  P1=1800.;
  P3= 250.;
  P5=  35.;
  P7=   5.;
  P9=   0.8122;
  F3=0.12*F1;
  F5=0.08*F1;
  F7=0.04*F1;

/* iteratively solve system */

  printf("it   F3  /  F4      F5 /  F6     F7 / F8      P9 /  P10\n");
  for(iter=0;iter<32;iter++)
    {
    H1=HofTP(T1,P1,year);                           /* boiler exit */
    P2=P1*(1.-dP12);                                /* main steam at turbine */
    H2=H1-dH12;
    T2=TofPH(P2,H2,year);
    Turbine(P2,H2,P3,STeff(F1),&H3);                /* first stage */
    Turbine(P3,H3,P5,STeff(F1),&H5);                /* second stage */
    Turbine(P5,H5,P7,STeff(F1),&H7);                /* third stage */
    Turbine(P7,H7,P9,STeff(F1),&H9);                /* ELEP [BTU/lbm] */
    F9=F1-F3-F5-F7;
    Van=AnnulusVelocity(F9,P9,H9);                  /* annulus velocity [ft/sec] */
    H10=H9+Exloss(Van);                             /* UEEP [BTU/lbm] */
    P4=P3*(1.-dP34);
    H4=H3-dH34;
    T4=TofPH(P4,H4,year);                           /* 1st extraction */
    P6=P5*(1.-dP56);
    H6=H5-dH56;
    T6=TofPH(P6,H6,year);                           /* 2nd extraction */
    P8=P7*(1.-dP78);
    H8=H7-dH78;
    T8=TofPH(P8,H8,year);                           /* 3rd extraction */
    H11=Hf(Tsat(P9,year),year);                     /* condensate */
    F11=F9+F7;
    P12=P5*1.5;
    Pump(P9,H11,P12,0.75,&H12);                     /* pump */
    FWH(F11,P12,H12,P8,H8,TTD3,DCA3,&F8,&H13,&H14); /* FWH3 */
    Deaerator(F11,H14,P6,H6,&F6,&H15);              /* deaerator */
    P16=P1/(1.-dPbo);
    Pump(P6,H15,P16,0.65,&H16);                     /* pump */
    FWH(F1,P16,H16,P4,H4,TTD1,DCA1,&F4,&H17,&H18);  /* FWH1 */
    P10=Pcond(F8*(H13-H11)+F9*(H10-H11),T20);

/* check for convergence and exit loop or update and continue */

    printf("%2i %6.0lf/%6.0lf %6.0lf/%6.0lf %5.0lf/%5.0lf %6.4lf/%6.4lf\n",iter,F4,F3,F6,F5,F8,F7,P10,P9);
    converged=1;
    if(fabs(F4/F3-1.)>0.00001)
      converged=0;
    if(fabs(F6/F5-1.)>0.00001)
      converged=0;
    if(fabs(F8/F7-1.)>0.00001)
      converged=0;
    if(fabs(P10/P9-1.)>0.00001)
      converged=0;
    c2=ff1*F1;
    c3=ff2*(F1-F3);
    c5=ff3*(F1-F3-F5);
    c7=ff4*(F1-F3-F5-F7);
    if(fabs(P2/c2-1.)>0.00001)
      converged=0;
    if(fabs(P3/c3-1.)>0.00001)
      converged=0;
    if(fabs(P5/c5-1.)>0.00001)
      converged=0;
    if(fabs(P7/c7-1.)>0.00001)
      converged=0;
    if(converged)
      break;
    F3=F4;
    F5=F6;
    F7=F8;
    P9=P10;
    P1=c2/(1.-dP12);
    P3=c3;
    P5=c5;
    P7=c7;
    }

/* generator output and losses */

  kWgross=(F1*(H2-H3)+(F1-F3)*(H3-H5)+(F1-F3-F5)*(H5-H7)+(F1-F3-F5-F7)*(H7-H10))/3412.141633;
  kWnet=kWgross;
  for(iter=0;iter<5;iter++)
    kWnet=kWgross-GenLoss(kWnet,PF);

/* boiler efficiency and heat input */

  Qnet=F1*(H1-H18);
  Qgross=Qnet/BlrEff(F1)/1E6;

  printf("kWnet=%.0lf, Qgross=%.2lf MBTU/hr, HtRt=%.0lf BTU/kWh\n",kWnet,Qgross,1E6*Qgross/kWnet);
  }

extern int __stdcall QueryPerformanceFrequency(__int64*);
extern int __stdcall QueryPerformanceCounter(__int64*);

int main(int argc,char**argv,char**envp)
  {
  int cases;
  double F1,F20=7750000.,PF,T1,T20;
  __int64 rate,t1,t2;
  if(argc>1)
    year=atoi(argv[1]);
  QueryPerformanceFrequency(&rate);
  QueryPerformanceCounter(&t1);
  for(cases=0,PF=0.85;PF<1.01;PF+=0.05)
    {
    for(T20=40.;T20<91.;T20+=10.)
      for(F1=500000.;F1<1250001.;F1+=250000.)
        for(T1=950.;T1<1051.;T1+=25.,cases++)
          SolveCycle(F1,T1,F20,T20,PF);
    fprintf(stderr,"PF=%4.2lf\n",PF);
    }
  QueryPerformanceCounter(&t2);
  printf("%i cases in %.3lf seconds\n",cases,(t2-t1)/((double)rate));
  fprintf(stderr,"%i cases in %.3lf seconds\n",cases,(t2-t1)/((double)rate));
  return(0);
  }
  